/*******************************************************************************
                                	REAL TIME ALARM CLOCK
                               		====================

This project is about designing an accurate real time alarm clock using the PCF8583 RTC chip

The PCF8583 chip is connected to the I2C pins (modul 1) of a PIC18F45K22 microcontroller.
The connections between the PCF8583 and the microcontroller are as follows The SDA, SCL
and INT lines are pulled high with resistors):

 PCF8583  Microcontroller
 =======  ===============
   SDA        SDA1 (RC4)
   SCL        SCL1 (RC3)
   INT        RC2
 
Clock timing to the PCF8583 is provided with a 32.768kHz crystal. In this project both the
clock and daily alarms can be set. When an alarm occurs, an LED connected to pin RD7 is
turned ON.

  MODE (RD0):            	Enter clock setup mode. Also change fields during setup
  UP (RD1)  :           	Increments the value
  DOWN (RD2):            	Decrements the value
  ALARM SETUP (RD3):    	Enter Alarm setup mode
  STOP ALARM (RD4):      	Stop present alarm condition

AN LCD is connected to PORTB of the microcontroller to help in setting the clock and alarm
time and also for displaying the clock data in real time.

The microcontroller operated with an 8MHz crystal with the PLL disabled (i.e. the actual
running clock frequency is 8MHz).

Author:   Dogan Ibrahim
Date:     October 2013
File:     MIKROC-I2C2.C
*******************************************************************************/
// LCD module connections
sbit LCD_RS at LATB4_bit;
sbit LCD_EN at LATB5_bit;
sbit LCD_D4 at LATB0_bit;
sbit LCD_D5 at LATB1_bit;
sbit LCD_D6 at LATB2_bit;
sbit LCD_D7 at LATB3_bit;

sbit LCD_RS_Direction at TRISB4_bit;
sbit LCD_EN_Direction at TRISB5_bit;
sbit LCD_D4_Direction at TRISB0_bit;
sbit LCD_D5_Direction at TRISB1_bit;
sbit LCD_D6_Direction at TRISB2_bit;
sbit LCD_D7_Direction at TRISB3_bit;
// End LCD module connections

#define MODE PORTD.RD0              			// MODE button
#define UP PORTD.RD1                			// UP button
#define DOWN PORTD.RD2              			// DOWN button
#define ALARM_SETUP PORTD.RD3       			// ALARM SETUP button
#define STOP_ALARM PORTD.RD4        			// Alarm stop button
#define ALARM_INT PORTC.RC2         			// RTC clock alarm INT pin
#define ALARM_LED PORTD.RD7         			// LED connected to RD7
#define SETUP 0


unsigned char seconds, minutes, hours, day, month, year;
unsigned char newday, newmonth, newyear, newhour, newminutes, newseconds;
//
// Ths function reads the Date and Time from the RTC chip
//
void Read_Date_Time()
{
    I2C1_Start();                  				// Send START bit to RTC chip
    I2C1_Wr(0xA0);                 				// Address the RTC chip
    I2C1_Wr(0x02);                 				// Start from address 2 (seconds)
    I2C1_Repeated_Start();         				// Issue repeated START bit
    I2C1_Wr(0xA1);                 				// Address the RTC chip for reading
    seconds = I2C1_Rd(1);          				// Read seconds, send ack
    minutes = I2C1_Rd(1);          				// Read minutes, send ack
    hours = I2C1_Rd(1);            				// Read hours, send ack
    day = I2C1_Rd(1);              				// Read year/day, send ack
    month = I2C1_Rd(0);            				// Read month, no ack (last byte to read)
    I2C1_Stop();                   				// Send STOP bit to RTC chip
    
}


//
// This function convert the date-time into correct format for displaying on the LCD. The numbers
// in RTC memory are in BCD form and are converted as follows:
//    "extract upper byte, shift right 4 bits, multiply by 10, add lower byte".
// For example, number 25 in RTC memory is stored as 37. i.e bit pattern: "0010 0101". After the
// conversion we obtain the required number 25.
//
Convert_Date_Time()
{
    seconds = ((seconds & 0xF0) >> 4)*10 + (seconds & 0x0F);
    minutes = ((minutes & 0xF0) >> 4)*10 + (minutes & 0x0F);
    hours =  ((hours & 0xF0) >> 4)*10 + (hours & 0x0F);
    month = ((month & 0x10) >> 4)*10 + (month & 0x0F);;
    year = (day & 0xC0) >> 6;
    day = ((day & 0x30) >> 4)*10 + (day & 0x0F);
}

//
// Display the date and time on the LCD
//
void Display_Date_Time()
{
//
// Write day, month, year as: dd=mm=xxxy
//
     Lcd_Chr(1, 1, (day / 10) + '0');
     Lcd_Chr(1, 2, (day % 10) + '0');
     Lcd_Chr(1, 4, (month / 10) + '0');
     Lcd_Chr(1, 5, (month % 10) + '0');
     Lcd_Chr(1 , 10, year + '0');
//
// Write hour, minutes, seconds as: hh:mm:ss
//
     Lcd_Chr(2, 1, (hours / 10) + '0');
     Lcd_Chr(2, 2, (hours % 10) + '0');
     Lcd_Chr(2, 4, (minutes / 10) + '0');
     Lcd_Chr(2, 5, (minutes % 10) + '0');
     Lcd_Chr(2, 7, (seconds / 10) + '0');
     Lcd_Chr(2, 8, (seconds % 10) + '0');
}

//
// This function gets the date and Time from the user via the 3 buttons. New values of Date
// and Time are returned to the calling program. Initially the maximum values are shown
// and these can be changed using the UP and DOWN buttons.
//
unsigned char Set_Date_Time(unsigned char *str, unsigned char min, unsigned char max)
{
     unsigned char c, Txt[4];
     
     ByteToStr(max, Txt);                 		// Convert maximum value to string in Txt
     Lcd_Cmd(_LCD_CLEAR);
     Lcd_Out(1,1,str);                   		// Display field name (e.g. "DAY:")
     Lcd_Out(2,1,Txt);                   		// Display maximum value to start with
     c = max;
     
     while(MODE == 1)                     		// While MODE button is not pressed
     {
        if(UP == 0)                       		// If UP button is pressed (increment)
        {
           Delay_Ms(10);
           while(UP == 0);                  		// Wait until UP button is released
           c++;                              		// Increment value
           if(c > max)c = min;                		// If greater than max, rollover to min
        }
        if(DOWN == 0)                      		// If DOWN button is pressed
        {
           Delay_Ms(10);
           while(DOWN == 0);                		// Wait until DOWN button is released
           c--;                                         // Decrement value
           if(c < min || c == 255)c = max;          	// If less than min, rollower to max
        }
        ByteToStr(c, Txt);                          	// Convert selected value to string
        Lcd_Out(2,1,Txt);                             	// Display selected value on LCD
     } 
     Delay_Ms(10);
     while(MODE == 0);                             	// Wait until MODE button is released
     return c;                                      	// Return selected number to calling program
}


//
// This function sets the RTC with the new Date and Time values. The number to be sent to
// the RTC chip is divided by 10, shifted left 4 digits, and the remainder is added to it. Thus
// for example if the number is decimal 25, it is converted into bit pattern  "0010 0101" and
// stored at the appropriate RTC memory
//
void Set_RTC()
{
//
// Convert Date and Time into a format compatible with the RTC chip
//
    seconds = ((newseconds / 10) << 4) + (newseconds % 10);
    minutes = ((newminutes / 10) << 4) + (newminutes % 10);
    hours = ((newhour / 10) << 4) + (newhour % 10);
    month = ((newmonth / 10) << 4) + (newmonth % 10);
    day = (newyear << 6) + ((newday / 10) << 4) + (newday % 10);
     
    I2C1_Start();              				// Send START bit to RTC chip
    I2C1_Wr(0xA0);             				// Address the RTC chip
    I2C1_Wr(0x00);             				// Start from address 0
    I2C1_Wr(0x80);             				// Pause RTC counter 
    I2C1_Wr(0x00);             				// Write to hundreths memory location
    I2C1_Wr(seconds);          				// Write to seconds memory location
    I2C1_Wr(minutes);          				// Write to minutes memory location
    I2C1_Wr(hours);            				// Write to hours memory location;
    I2C1_Wr(day);              				// Write to year/day memory location
    I2C1_Wr(month);            				// Write to month memory location
    I2C1_Stop();               				// Send STOP bit
    
    I2C1_Start();              				// Send START bit to RTC chip
    I2C1_Wr(0xA0);             				// Address the RTC chip
    I2C1_Wr(0);                				// Start from address 0 (Configuration reg)
    I2C1_Wr(0);                				// Write 0 to Configuration reg to start counter
    I2C1_Stop();               				// Send STOP bit
}

//
// This function sets the RTC alarm. The alarm is configured to occur every day
// at the set time. The BUZZER sounds when the alarm occurs
//
void Set_RTC_Alarm()
{
//
// Convert Alarm Time into a format compatible with the RTC chip. For daily Alarm
// setup the Date fields (day, month, year) are ignored
//
    seconds = ((newseconds / 10) << 4) + (newseconds % 10);
    minutes = ((newminutes / 10) << 4) + (newminutes % 10);
    hours = ((newhour / 10) << 4) + (newhour % 10);

    I2C1_Start();              				// Send START bit to RTC chip
    I2C1_Wr(0xA0);             				// Address the RTC chip
    I2C1_Wr(0x00);             				// Start from address 0
    I2C1_Wr(0x04);             				// Enable Alarm Control register
    I2C1_Stop();
    
    I2C1_Start();
    I2C1_Wr(0xA0);             				// Address the RTC chip
    I2C1_Wr(0x08);             				// Start from address 8
    I2C1_Wr(0x90);             				// Enable Daily alarms, enable INT output
    I2C1_Wr(0x00);             				// Write to hundreths memory location
    I2C1_Wr(seconds);          				// Write to seconds memory location
    I2C1_Wr(minutes);          				// Write to minutes memory location
    I2C1_Wr(hours);            				// Write to hours memory location;
    I2C1_Stop();              				 // Send STOP bit
}

//
// This function stops the alarm by clearing the timer register of the
// control and status register
//
void Reset_Alarm_Flag()
{
    I2C1_Start();              				// Send START bit to RTC chip
    I2C1_Wr(0xA0);             				// Address the RTC chip
    I2C1_Wr(0x00);             				// Start from address 0
    I2C1_Wr(0x04);             				// Reset Alarm flag
    I2C1_Stop();               				// Send STOP bit
}


void main()
{
    ANSELB = 0;                              		// Configure PORTB as digital
    ANSELC = 0;                              		// Configure PORTC as digital
    ANSELD = 0;                              		// Configure PORTD as digital
    TRISC.RC2 = 1;                           		// Alarm INT input
    TRISD.RD0 = 1;                           		// Configure RD0 (MODE)as input
    TRISD.RD1 = 1;                           		// Configure RD1 (UP) as input
    TRISD.RD2 = 1;                           		// Configure RD2 (DOWN) as input
    TRISD.RD3 = 1;                           		// Configure RD3 (Alarm setup)
    TRISD.RD4 = 1;                           		// Configure RD4 as input (Alarm STOP)
    TRISD.RD7 = 0;                           		// Alarm LED is output
    
    ALARM_LED = 0;                           		// Alarm LEDOFF to start with
    Lcd_Init();                              		// Initialize LCD
    Lcd_Cmd(_LCD_CURSOR_OFF);                		// Disable cursor
    Lcd_Cmd(_LCD_CLEAR);                     		// Clear LCD
    
    
    I2C1_Init(100000);                        		// Initialize I2C module
//
// If the MODE button is pressed on startup we must get into SETUP phase
//
    if(MODE == SETUP)                       		// If CLOCK SETUP mode
    {
        while(MODE == SETUP);                  		// Wait until MODE button is released
        newday = Set_Date_Time("DAY:",1,31);                     // Get current day
        newmonth = Set_Date_Time("MONTH:", 1,12);                // Get current month
        newyear = Set_Date_Time("YEAR (201x):", 3,6);            // Get current year (201x)
        newhour = Set_Date_Time("HOUR:", 0, 23);                 // Get current hour
        newminutes = Set_Date_Time("MINUTES:", 0, 59);           // Get current minutes
        newseconds = Set_Date_Time("SECONDS:", 0, 59);           // Get current seconds
        //
        // We have got all the new Date and Time values. Now set the RTC with these values
        //
        Set_RTC();
    }
    if(ALARM_SETUP == 0)                          		// If ALARM SETUP mode
    {
        while(ALARM_SETUP = 0);                   		// Wait until ALARM button is released
        newhour = Set_Date_Time("ALRM HOUR:", 0, 23);
        newminutes = Set_Date_Time("ALRM MINS:", 0, 59);
        newseconds = Set_Date_Time("ALRM SECS:", 0, 59);
        //
        // We have got the Daily Alarm time. Now set the RTC clock to generate alarm
        // at this time every day. Sound the BUZZER when alarm is generated.
        //
        Set_RTC_Alarm();
    }
        

//
// Read the Date and Time from the RTC chip and display on the LCD in the following format:
// Row 1: dd-mm-yyyy
// Row 2: hh:mm:ss
//
    Lcd_Out(1, 1, "dd-mm-2013");
    Lcd_Out(2, 1, "hh:mm:ss");
    while(1)
    {
        Read_Date_Time();                       		// Read Date and Time from RTC chip
        Convert_Date_Time();                    		// Convert into a form to display
        Display_Date_Time();                    		// Display Date and Time on the LCD

        if(ALARM_INT == 0)                     			// If alarm occurred (active LOW)
        {
             ALARM_LED = 1;                     		// Turn ON ALARM LED
             if(STOP_ALARM == 0)                		// if STOP ALARM button pressed
             {
                ALARM_LED = 0;                   		// Turn OFF ALARM LED
                Reset_Alarm_Flag();                  		// Reset alarm flag back to 1
             }
        }
    }
}
