/************************************************************************
                  	FINITE IMPULSE RESPONSE FILTER DESIGN
                   	=====================================

This project shows how a FIR type digital filter can be designed.

Analog sine wave signal is fed to the AN0 (RA0) analog input of the microcontroller.
A D/A converter is connected to the microcontroller through the SPI bus so that the
filtered signal is in analog form and can be plotted using a frequency plotter.

In this example a LOW-Pass filter is designed with the following specifications:

        Filter Type:                            No windowing
        Sampling Frequency:                  	1000Hz
        Passband Upper Frequency:            	50Hz
        Filter Order:                        	10 (11 taps)

The FIR filter coefficients are obtained using the ScopeFIR software package.

The filter response is plotted using a Velleman PCSGU250 type oscilloscope+
frequency generator + frequency plotter device.

The PIC18F45K22 microcontroller is used in this project. The external clock is supplied
using an 8MHz crystal, but internally the PLL is used to increase the clock frequency
to 32MHz (this is done during the programming by enabling 4xPLL and setting the
clock frequency to 32MHz in Project -> Edit Project window).

The D/A converter used is the MCP4921 SPI bus based converter with 12-bit resolution, 
operating with +5V reference voltage.

The connection between the microcontroller and the D/A converter is as follows:

Author:      	Dogan Ibrahim
Date:        	October 2013
File:        	MIKROC-FIR.C
*********************************************************************************/
// DAC module connections
sbit Chip_Select at LATE0_bit;
sbit Chip_Select_Direction at TRISE0_bit;
// End DAC module connections

#define N 10                             			// Filter order=10, having 11 taps

float Sample,xn, yn, x[N];
unsigned ADC;
unsigned char temp;
unsigned int DAC;
float h[N+1] = {0.0681, 0.0810, 0.0918, 0.1001, 0.1052, 0.1070, 0.1052, 0.1001, 0.0918, 0.0810, 0.0681};
//
// Timer 1 interrupt service routine. The program jumps here every 1000us (the sampling 
// frequency is 1kHz, i.e. Period = 1ms = 1000us). Here, a new output is calculated and sent
// to the D/A converter
//
void interrupt()
{
     unsigned char i;

     TMR0L = 6;                                    		// Re-load TMR0
     ADC = ADC_Get_Sample(0);                      		// Get new input Sample from AN0
     x[0]=ADC;
     yn = 0.0;          
//
// Calculate a new output yn
//
     for(i = 0; i <= N; i++)
     {
       yn = yn + h[i]*x[i];
     }
//
// Output the new Sample via the D/A converter
//
     DAC = yn;
     Chip_Select = 0;                              		// Select DAC chip

     // Send High Byte
     temp = (DAC >> 8) & 0x0F;                     		// Store DAC[11..8] to temp[3..0]
     temp |= 0x30;                                 		// Define D/A setting
     SPI1_Write(temp);                             		// Send high byte via SPI

     // Send Low Byte
     temp = DAC;                                   		// Store DAC[7..0] to temp[7..0]
     SPI1_Write(temp);                             		// Send low byte via SPI

     Chip_Select = 1;                             		 // Deselect D/A converter chip
//
// Shift the input samples for the delay action
//
     for(i = 0; i < N; i++)
     {
       x[N-i] = x[N-i-1];
     }
//
// Re-enable Timer 0 interrupts
//
     INTCON.TMR0IF = 0;                            		// Clear Timer 0 interrupt flag
}


//
//************************* MAIN PROGRAM **********************
// Start of MAIN program. In the main program the I/O ports are configured, SPI bus
// library is initialized, and the A/D converter module library is initialized. In addition,
// Timer 0 is configured to interrupt at every one millisecond and interrupts are enabled.
//
void main()
{
      TRISA0_bit = 1;                              		// AN0 (RA0) is input
      ANSELE = 0;                                  		// RE0 is digital I/O
      ANSELA = 1;                                  		// RA0 is analog I/O
      Chip_Select_Direction = 0;                   		// Configure CS pin as output
      Chip_Select = 1;                             		// Disable D/A converter
      SPI1_Init();                                 		// Initialize SPI1
      ADC_Init();                                  		// Initialize A/D converter

//
// Configure Timer 0 for 1000us (1ms) interrupts
//
      T0CON = 0x44;                                		// Disable TMR0, 8-bit, prescaler-32
      INTCON.TMR0IF = 0;                           		// Clear Timer 0 interrupt flag
      INTCON.TMR0IE = 1;                           		// Enable Timer 0 interrupts
      TMR0L = 6;                                   		// Load TMR0
      T0CON.TMR0ON = 1;                            		// Enable Timer 0
      INTCON.GIE = 1;                              		// Enable global interrupts

      for(;;)                                      		// Wait for Timer 0 interrupts
      {
      }
}

