/******************************************************************************
                           		Frequency Counter
                           		================

This project is a frequency counter. The signal whose frequency is to be measured is applied
to pin RA4 (T0CKI) of a PIC18F45K22 microcontroller, operating from an 8MHz crystal. The
project measures the frequency and displays on an  LCD in Hz.

The resolution of the measurement is 1Hz. The project can measure frequencies from 1Hz to
several MHz.

The project uses 2 timers, TIMER0 and TIMER1. TIMER1 is usd to open a time window of 1
second width. The pulses of the external signal increment the counter during this 1 second
window. At the end of 1 second both timers are stopped. The count in TIMER0 gives us
directly the frequency in Hz.

Both timers generate interrupts for higher accuracy. TIMER0 uses a variable called "Overflow"
to find out how many times it has overflowed (if any). This variable is used in calculating the
 total count. TIMER1 interrupts at every 250ms and a variable called Cnt is incremented at
each interrupt. When Cnt = 4 then it is assumed that 1 second has elapsed.

Programmer:     Dogan Ibrahim
Date:           September 2013
File:           MIKROC-FREQ1.C

*******************************************************************************/
// LCD module connections
sbit LCD_RS at RB4_bit;
sbit LCD_EN at RB5_bit;
sbit LCD_D4 at RB0_bit;
sbit LCD_D5 at RB1_bit;
sbit LCD_D6 at RB2_bit;
sbit LCD_D7 at RB3_bit;

sbit LCD_RS_Direction at TRISB4_bit;
sbit LCD_EN_Direction at TRISB5_bit;
sbit LCD_D4_Direction at TRISB0_bit;
sbit LCD_D5_Direction at TRISB1_bit;
sbit LCD_D6_Direction at TRISB2_bit;
sbit LCD_D7_Direction at TRISB3_bit;
// End LCD module connections

#define PULSE PORTA.RA4

unsigned int Overflow;
unsigned char Cnt;
//
// Timr interrupt service routine. Program jumps here at every 10ms
//
void interrupt (void)
{
     if(INTCON.TMR0IF == 1)                                    // If TIMER0 interrupt
     {
        Overflow++;                                            // Increment Overflow count
        INTCON.TMR0IF = 0;                                     // Clear Timer0 interrupt flag
     }
     if(PIR1.TMR1IF == 1)                                      // If TIMER1 interrupt
     {
       TMR1H = 0x0B;                                           	// Re-load timer register
       TMR1L = 0xDC;
       Cnt++;                                                  	// Increment Cnt
       PIR1.TMR1IF = 0;                                         // Clear Timer1 interrupt flag
     }
}


void main()
{
     unsigned char Txt[11];
     unsigned long Elapsed;
     unsigned char L_Byte, H_Byte;
     
     ANSELA = 0;                                         	// Configure PORTA as digital
     ANSELB = 0;
     TRISA.RA4 = 1;                                       	// RA4 is input
     Lcd_Init();                                          	// Initialize LCD
     Lcd_Cmd(_LCD_CURSOR_OFF);                            	//  Disable cursor

//
 // Configure TIMER0 for 16-bit mode, no prescaler, clock provided by external
 // signal into pin T0CKI. Timer is not started here
 //
     T0CON = 0x28;                                      	// TIMER0 16-bit,no prescaler,T0CKI clk
//
// Configure Timer1 for 250ms Overflow. Timer1 is not started here
//
     T1CON = 0x36;
     
     PIE1 = 0x01;                                       	// Enable TIMER1 interrupts
     PIR1.TMR1IF = 0;                                   	// Clear TIMER1 interrupt flag
     INTCON = 0xE0;                                     	// Enable TIMER0 and TIMER1 interrupts
     
     for(;;)                                            	// Wait for interrupts
     {
       TMR0H = 0;                                       	// Clear Timer0 registers
       TMR0L = 0;
       TMR1H = 0x0B;                                    	// Load Timer1 registers
       TMR1L = 0xDC;
       //
       Overflow = 0;                                    	// Clear Overflow count
       Cnt = 0;                                         	// Clear Cnt
       //
       // Start TIMER0. It will increment each time an external pulse is detected.
       // TIMER0 increments on the rising edge of the external clock
       //
       T0CON.TMR0ON = 1;
       //
       // Start Timer1 to count 4x250ms = 1 second
       //
       T1CON.TMR1ON = 1;
       while(Cnt != 4);                           		// Wait until 1 second has elapsed
       //
       // 1 second has elapsed. Stop both timers
       //
       T0CON.TMR0ON = 0;                             		// Stop TIMER0
       T1CON.TMR1ON = 0;                            		// Stop Timer1
       // Get TIMER0 count
       L_Byte = TMR0L;
       H_Byte = TMR0H;
       //
       // Store TIMER0 count is variable Elapsed
       //
       Elapsed = (unsigned long)256*H_Byte + L_Byte;
       Elapsed = 65536*Overflow + Elapsed;
       //
       // Convert into string and display
       //
       LongWordToStr(Elapsed, Txt);              		 // Long to string conversion
       Lcd_Cmd(_LCD_CLEAR);                                	 // Clear LCD
       Lcd_Out(1,1,"Frequency (HZ)");                    	 // Display heading
       Lcd_Out(2,1,Txt);                                	 // Display measured frequency
       
       Delay_ms(1000);                  			 // Wait 1 second and repeat
     }
}
