/*******************************************************************************
                   	BAROMETER, THERMOMETER AND ALTIMETER DISPLAY
                  		 			-------------------------------------------

This program uses the LPS331AP MEMS pressure sensor chip. In addition to pressure, the chip
also measures the temperature, and the altimeter reading can be obtained
from the pressure reading

The program displays the pressure, altitude, and temperature. The program works in One-shot
mode. i.e. a new sample is requested and the sample is received and displayed. Then a new
sample is requested and so on.

The clock PLL is enabled so that the actual clock frequency is 32MHz (Enable the 4xPLL and set oscillator frequency to 32MHz in Project -> Edit Project)

Author:  Dogan Ibrahim
Date:    October, 2013
File:    MIKROC-GLCD2.C
*******************************************************************************/

// Glcd module connections
char GLCD_DataPort at PORTD;

sbit GLCD_CS1 at LATB0_bit;
sbit GLCD_CS2 at LATB1_bit;
sbit GLCD_RS  at LATB2_bit;
sbit GLCD_RW  at LATB3_bit;
sbit GLCD_EN  at LATB4_bit;
sbit GLCD_RST at LATB5_bit;

sbit GLCD_CS1_Direction at TRISB0_bit;
sbit GLCD_CS2_Direction at TRISB1_bit;
sbit GLCD_RS_Direction  at TRISB2_bit;
sbit GLCD_RW_Direction  at TRISB3_bit;
sbit GLCD_EN_Direction  at TRISB4_bit;
sbit GLCD_RST_Direction at TRISB5_bit;
// End Glcd module connections

// Define LPS331AP registers
#define REF_P_XL 0x08                         		// Reference pressure (LSB)
#define REF_P_L 0x09                          		// Reference pressure (middle)
#define REF_P_H 0x0A                         		// Reference pressure (MSB)
#define WHO_AM_I 0x0F                         		// Device identification
#define RES_CONF 0x10                         		// Pressure resolution
#define CTRL_REG1 0x20                        		// Control register 1
#define CTRL_REG2 0x21                        		// Control register 2
#define CTRL_REG3 0x22                        		// Control register 3
#define INT_CFG_REG 0x23                      		// Interrupt configuration register
#define INT_SOURCE_REG 0x24                   		// Interrupt source register
#define THS_P_LOW_REG 0x25                    		// Threshold LOW register
#define THS_P_HIGH_REg 0x26                   		// Threshold HIGH register
#define STATUS_REG 0x27                       		// Status register
#define PRESS_POUT_XL_REH 0x28                    	// Pressure output register (LSB)
#define PRESS_OUT_L 0x29                      		// Pressure output register (middle)
#define PRESS_OUT_H 0x2A                      		// Pressure output register (MSB)
#define TEMP_OUT_L 0x2B                       		// Temperature output register (LSB)
#define TEMP_OUT_H 0x2C                       		// Temperature output register (MSB)
#define AMP_CTRL 0x30                         		// Analog front end control register
#define Write_Addr 0xBA                    		// Device write register
#define Read_Addr 0xBB                    		// Device read register

void Pressure_Write(unsigned char address, unsigned char value)
{
    I2C1_Start();                          		// Send START bit
    I2C1_Wr(Write_Addr);                   		// Send device address
    I2C1_Wr(address);                      		// Send register address
    I2C1_Wr(value);                        		// Send data
    I2C1_Stop();                           		// Send STOP bit
}

unsigned char Pressure_Read(unsigned char address)
{
    unsigned char c = 0;
    
    I2C1_Start();                     			// Send START bit
    I2C1_Wr(Write_Addr);              			// Send device address
    I2C1_Wr(address);                 			// Send register address
    I2C1_Repeated_Start();            			// Send repeated START
    I2C1_Wr(Read_Addr);               			// Send device read address
    c = I2C1_Rd(0);                   			// Read, send no ack
    I2C1_Stop();                      			// Send STOP bit
    return c;
}
//
// This function initializes the pressure chip
//
unsigned char Init_Pressure_Chip()
{
    unsigned char temp, flag = 0;

    Pressure_Write(RES_CONF, 0x78);           		// Select pressure and temp resolution
    Pressure_Write(CTRL_REG1, 0x04);          		// Configure One-shot mode
    Pressure_Write(CTRL_REG1, 0x84);        		// Configure chip active mode
    temp = Pressure_Read(WHO_AM_I);           		// Read chip identity
    if(temp != 0xBB)flag = 1;                 		// Error if wrong chip identity
    return flag;
}

//
// This function reads and returns the pressure (24 bits). The reading is converted into
// millibars after dividing by 4019 (shifting right 12 bits)
//
//
long int Read_Pressure_Value()
{
     long int outP;
     unsigned char stat, P_DA, PressureM, PressureL;
     
     Pressure_Write(CTRL_REG2, 1);                  		// Send One-shot request
//
// Check if new pressure data is available, if so read it
//
     P_DA = 0;
     while(P_DA == 0)                              		// Wait until new pressure is available
     {
        stat = Pressure_Read(STATUS_REG);          		 // Read the status register
        P_DA = stat & 0x02;                            		// Extract P_DA
     }
     
     OutP = Pressure_Read(PRESS_OUT_H);          		// Read high byte
     PressureM = Pressure_Read(PRESS_OUT_L);          		// Read middle byte
     PressureL = Pressure_Read(PRESS_POUT_XL_REH);         	// Read low byte

     OutP = (OutP << 8);                     			// Move to middle byte position
     OutP = OutP | PressureM;                			// Add middle byte
     OutP = (OutP << 8);                       			// Move to upper byte position
     OutP = (OutP | PressureL);                			// Add low byte
     OutP = (OutP  >> 12);                     			// Divide by 4096 (in mbars)
     return OutP;                            			// Return the pressure
}

//
// This function reads and returns the temperature (16 bits). The reading is converted into
// Degrees Centigrade after the following operation:
//
//    (Value / 480) + 42.5
//
//
float Read_Temperature_Value()
{
    int OutT;
    unsigned char stat, T_DA, TempL;
    float DegreesC;
//
// Wait until a new temperature data is available and if so get it
//
    T_DA = 0;
    while(T_DA == 0)                                     	// Wait for new temperature
    {
       stat = Pressure_Read(STATUS_REG);          		// Read status register
       T_DA = stat & 0x01;                           		// extract T_DA bit
    }
    
    OutT = Pressure_Read(TEMP_OUT_H);        			// Read high byte
    TempL = Pressure_Read(TEMP_OUT_L);       			// Read low byte
    
    OutT = OutT << 8;                        			// Move to left byte
    OutT = OutT | TempL;                     			// Add low byte
    DegreesC = ((OutT / 480.0) + 42.5);
    return DegreesC;                             		// Return the temperature
}

//
// This function calculates the height (altimeter function) from the pressure
//
float Read_Altimeter_Value(long int Pressure)
{  
   float Altitude_ft;
   
   Altitude_ft = Pressure/1013.25;
   Altitude_ft = pow(Altitude_ft, 0.190284);
   Altitude_ft = (1.0 - Altitude_ft)*145366.45;

   return Altitude_ft;
}

//
// This function displays the pressure, temperature and altitude on the GLCD display
// in the following format:
//
//   P(mb): nnnn
//   T(C) : nn.n
//   A(ft): nnn.n
//
void Display_PTA(long int Pressure, float Temperature, float Altitude)
{
     unsigned char i, Txt[14];
     char *res;
     
     Glcd_Rectangle(5,5,120,55,1);                              // Draw rectangle
     Glcd_Write_Text("P(mb): ", 7,1,1);                        	// Write string "Pressure (mb): "
     Glcd_Write_Text("T(C) : ", 7,3,1);
     Glcd_Write_Text("A(ft): ", 7,5,1);
// Display Pressure
     for(i=0; i<14; i++)Txt[i] = 0;
     LongWordToStr(Pressure, Txt);
     Ltrim(Txt);
     Glcd_Write_Text(Txt, 45,1,1);
// Display temperature
     for(i=0; i<14; i++)Txt[i] = 0;
     FloatToStr(Temperature, Txt);
     res = strrchr(Txt, '.');                                  // Locate "."
     *(res + 2) = 0x0;                                         // Terminate after 1 digit
     Glcd_Write_text(Txt, 45, 3, 1);
// Display altitude
     for(i=0; i<14; i++)Txt[i] = 0;
     FloatToStr(Altitude, Txt);
     res = strrchr(Txt, '.');
     *(res + 2) = 0x0;
     Glcd_Write_Text(Txt, 45, 5, 1);
}

//
// Display Error message
//
void Error()
{    
     Glcd_Write_Text("Error...", 5,3,1);           		// Wrong device ID
     while(1);
}


void main()
{
     unsigned char stat;
     long int P;
     float T, A;
     
     ANSELB = 0;                                        	// Configure PORTB as digital
     ANSELC = 0;                                        	// Configure PORTC as digital
     ANSELD = 0;                                        	// Configure PORTD as digital
     
     Glcd_Init();                                      		// Initialize GLCD
     Glcd_Fill(0x00);                                  		// Clear GLCD

     I2C1_Init(100000);                                		// Initialize I2C
     Delay_Ms(10);
     
     stat = Init_Pressure_Chip();                             	// Initialize pressure chip
     if(stat != 0)Error();
//
// Loop to read and display the pressure, temperature and altitude
//
     while(1)
     {
        P = Read_Pressure_Value();                   		// Read pressure
        T = Read_Temperature_Value();                		// Read temperature
        A = Read_Altimeter_Value(P);                  		// Read altitude
        Display_PTA(P, T, A);                         		// Display Pressure, Temp, Altitude
        Delay_Ms(5000);                               		// Wait 5 seconds
        Glcd_Fill(0x00);                             		// Clear GLCD
     }
}
