/*******************************************************************************
                  		TEMPERATURE PLOTTING ON GLCD
                  		============================

This project shows how the temperature can be read from an analog temperature sensor and then
plotted on a GLCD in real time.

In this project an LM35DZ type analog temperature sensor is used. This sensor has 3 pins: The
ground, power supply (+5V), and the output pin. The sensor gives an output voltage which is
directly proportional to the measured temperature. i.e. V o = 10mV/C. Thus, for example at 15C
the output voltage is 150mV. Similarly, at 30C the output voltage is 300mV and so on.

The temperature sensor is connected to analog input RA0 (or AN0) of a PIC18F45K22 type
microcontroller. The microcontroller is operated from an 8MHz crystal, with the PLL is disabled,
so that the actual clock frequency is 8MHz. The GLCD used in the project is based on KS0107/108 type controller with 128x64 pixels.

The program first draws the X and Y axes, axes ticks, and the Y axis labels. Then, the temperature
is read from Channel 0 (RA0 or AN0), converted into digital, and then into Degrees C. The
temperature is plotted in real-time every second. i.e. the horizontal axis is the time where each
pixel corresponds to 1 second.

The GLCD is connected to PORTB and PORTD of the microcontroller as in the previous
GLCD projects.

Author:  Dogan Ibrahim
Date:    October, 2013
File:    MIKROC-GLCD3.C
****************************************************************************/

unsigned char stp, old_x, old_y, new_x, new_y;

// Glcd module connections
char GLCD_DataPort at PORTD;

sbit GLCD_CS1 at RB0_bit;
sbit GLCD_CS2 at RB1_bit;
sbit GLCD_RS  at RB2_bit;
sbit GLCD_RW  at RB3_bit;
sbit GLCD_EN  at RB4_bit;
sbit GLCD_RST at RB5_bit;

sbit GLCD_CS1_Direction at TRISB0_bit;
sbit GLCD_CS2_Direction at TRISB1_bit;
sbit GLCD_RS_Direction  at TRISB2_bit;
sbit GLCD_RW_Direction  at TRISB3_bit;
sbit GLCD_EN_Direction  at TRISB4_bit;
sbit GLCD_RST_Direction at TRISB5_bit;
// End Glcd module connections


//
// This function plots the X and Y axis. The origin is set at screen co-ordinates (12,60).
// First the two axes are drawn. Then the axes ticks are displayed for both X and Y axis.
// Finally, the Y axis labels are displayed (i.e. the temperature labels)
//
void PlotAxis()
{
   unsigned char i;

   Glcd_Line(12, 0, 12, 60, 1);            				// Draw Y axis
   Glcd_Line(12, 60, 127, 60, 1);          				// Draw X axis
   for(i=12; i<127; i += 9)Glcd_Dot(i, 61, 1);  			// Display x axis ticks
   for(i=0; i<60; i += 10)Glcd_Dot(11, i, 1);    			// Display y axis ticks
   Glcd_Write_Text_Adv("30",0,5);               			// Y axis label
   Glcd_Write_Text_Adv("25",0,15);               			// Y axis label
   Glcd_Write_Text_Adv("20",0,25);              			// Y axis label
   Glcd_Write_Text_Adv("15",0,35);              			// Y axis label
   Glcd_Write_Text_Adv("10",0,45);               			// Y axis label
}

//
// This function plots the temperature in real-time. The temperature is plotted by joining
// the data points with straight lines. The X axis is the time where each pixel corresponds
// to one second. The Y axis is the temperature in Degrees C
//
void PlotXY(float Temperature)
{
   Glcd_Line(old_x,old_y,new_x,new_y,1);			// Draw temperature changes
   old_x = new_x;						// Update old points
   old_y = new_y;
}


//
// Start of main program
//
void main()
{
   unsigned int T;
   unsigned char flag = 0;
   float mV, C;

   ANSELA = 1;                          			// Configure PORTA as analog
   ANSELB = 0;                          			// Configure PORTB as digital
   ANSELD = 0;                           			// Configure PORTD as digital
   TRISA = 1;                            			// RA0 is input (analog)
   TRISB = 0;                            			// PORT B is output
   TRISD = 0;                          				// PORT D is output

   Glcd_Init();                               			// Initialise GLCD
   Glcd_Fill(0x0);                             			// Clear GLCD
   ADC_Init();                                 			// Initialise ADC
   PlotAxis();                                    		// Plot X-Y axes and labels

    for(;;)							// DO FOREVER
    {
       T = ADC_Get_Sample(0);                       		// Read temperature from channel 0
       mV = T*5000.0/1024.0;                        		// Temperature in mV
       C = mV /10.0;                               		// Temperature in Degrees C

       if(flag == 0)                                  		// If first time
       {
           new_x = 12;                                    	// Start from x = 12
           old_x = new_x;
           new_y = -2*C+70;                           		// New temperature value
           old_y = new_y;
           flag = 1;                                		// Set so that not first time
       }
       else                                        		// Not first time
       {
            new_x++;                                  		// Inc x by 1 (1 second each pixel)
            new_y = -2*C+70;                          		// New temperature value
       }
       PlotXY(C);                                  		// Plot the graph
       Delay_Ms(1000);                              		// Wait one second
     }
}

