/*******************************************************************************
                                	REAL TIME CLOCK
                                	===============

This project is about designing an accurate real time clock (RTC) using the RTC chip PCF8583

The PCF8583 chip is connected to the I2C pins (modul e 1) of a PIC18F45K22 microcontroller.
The connections between the PCF8583 and the microcontroller are as follows (the SDA and SCL
lines are pulled high with resistors):

 PCF8583  	Microcontroller
 =======  	=============
   SDA       	    SDA1 (RC4)
   SCL       	    SCL1 (RC3)
 
Clock timing to the PCF8583 is provided with a 32.768kHz crystal. 3 Push-button switches are
used to set the clock initially:

  	MODE (RD0): 	Selects the date or time field to be set
  	UP (RD1)  :	Increments the value
  	DOWN (RD2): 	Decrements the value

AN LCD is connected to PORTB of the microcontroller to help in setting the clock and also for
displaying the clock data in real time.

The microcontroller is operated with an 8MHz crystal with the PLL disabled (i.e. the actual
running clock frequency is 8MHz).

The software has 2 phases: SETUP and RUNNING. The SETUP phase is entered if the MODE
button is pressed during the startup. In this phase the clock is set to current date and time.
The RUNNING phase is entered if the MODE button is not pressed during the startup and
this is the normal running state of the clock where the date and time are displayed on the
LCD in the following format:

    	dd-mm-yyyy
    	hh:mm:ss

Author:   Dogan Ibrahim
Date:     October 2013
File:     MIKROC-I2C.C
*******************************************************************************/
// LCD module connections
sbit LCD_RS at LATB4_bit;
sbit LCD_EN at LATB5_bit;
sbit LCD_D4 at LATB0_bit;
sbit LCD_D5 at LATB1_bit;
sbit LCD_D6 at LATB2_bit;
sbit LCD_D7 at LATB3_bit;

sbit LCD_RS_Direction at TRISB4_bit;
sbit LCD_EN_Direction at TRISB5_bit;
sbit LCD_D4_Direction at TRISB0_bit;
sbit LCD_D5_Direction at TRISB1_bit;
sbit LCD_D6_Direction at TRISB2_bit;
sbit LCD_D7_Direction at TRISB3_bit;
// End LCD module connections

#define MODE PORTD.RD0            				// MODE button
#define UP PORTD.RD1         					// UP button
#define DOWN PORTD.RD2       					// DOWN button
#define SETUP 0

unsigned char seconds, minutes, hours, day, month, year;
unsigned char newday, newmonth, newyear, newhour, newminutes, newseconds;
//
// Ths function reads the Date and Time from the RTC chip
//
void Read_Date_Time()
{
    I2C1_Start();                  				// Send START bit to RTC chip
    I2C1_Wr(0xA0);                 				// Address the RTC chip
    I2C1_Wr(0x02);                 				// Start from address 2 (seconds)
    I2C1_Repeated_Start();         				// Issue repeated START bit
    I2C1_Wr(0xA1);                 				// Address the RTC chip for reading
    seconds = I2C1_Rd(1);          				// Read seconds, send ack
    minutes = I2C1_Rd(1);          				// Read minutes, send ack
    hours = I2C1_Rd(1);            				// Read hours, send ack
    day = I2C1_Rd(1);              				// Read year/day, send ack
    month = I2C1_Rd(0);            				// Read month, no ack (last byte)
    I2C1_Stop();                   				// Send STOP bit to RTC chip
    
}


//
// This function convert the date-time into correct format for displaying on the LCD. The
// numbers in RTC memory are in BCD form and are converted as follows:
//    "extract upper byte, shift right 4 bits, multiply by 10, add lower byte".
// For example, number 25 in RTC memory is stored as 37. i.e bit pattern: "0010 0101".
// After the conversion we obtain the required number 25.
//
Convert_Date_Time()
{
    seconds = ((seconds & 0xF0) >> 4)*10 + (seconds & 0x0F);
    minutes = ((minutes & 0xF0) >> 4)*10 + (minutes & 0x0F);
    hours =  ((hours & 0xF0) >> 4)*10 + (hours & 0x0F);
    month = ((month & 0x10) >> 4)*10 + (month & 0x0F);;
    year = (day & 0xC0) >> 6;
    day = ((day & 0x30) >> 4)*10 + (day & 0x0F);
}

//
// Display the date and time on the LCD
//
void Display_Date_Time()
{
//
// Write day, month, year as: dd=mm=xxxy
//
     Lcd_Chr(1, 1, (day / 10) + '0');
     Lcd_Chr(1, 2, (day % 10) + '0');
     Lcd_Chr(1, 4, (month / 10) + '0');
     Lcd_Chr(1, 5, (month % 10) + '0');
     Lcd_Chr(1 , 10, year + '0');
//
// Write hour, minutes, seconds as: hh:mm:ss
//
     Lcd_Chr(2, 1, (hours / 10) + '0');
     Lcd_Chr(2, 2, (hours % 10) + '0');
     Lcd_Chr(2, 4, (minutes / 10) + '0');
     Lcd_Chr(2, 5, (minutes % 10) + '0');
     Lcd_Chr(2, 7, (seconds / 10) + '0');
     Lcd_Chr(2, 8, (seconds % 10) + '0');
}

//
// This function gets the date and Time from the user via the 3 buttons. New values
// of Date and Time are returned to the calling program. Initially the maximum values
// are shown and these can be changed using the UP and DOWN buttons.
//
unsigned char Set_Date_Time(unsigned char *str, unsigned char min, unsigned char max)
{
     unsigned char c, Txt[4];
     
     ByteToStr(max, Txt);                 			// Convert max value to string in Txt
     Lcd_Cmd(_LCD_CLEAR);
     Lcd_Out(1,1,str);                   			// Display field name (e.g. "DAY:")
     Lcd_Out(2,1,Txt);                   			// Display max value to start with
     c = max;
     
     while(MODE == 1)                     			// While MODE button not pressed
     {
        if(UP == 0)                       			// If UP button pressed (increment)
        {
           Delay_Ms(10);
           while(UP == 0);                  			// Wait until UP button is released
           c++;                              			// Increment value
           if(c > max)c = min;                			// If greater than max rollover to min
        }
        if(DOWN == 0)                      			// If DOWN button is pressed
        {
           Delay_Ms(10);
           while(DOWN == 0);                			// Wait until DOWN button is released
           c--;                                            	// Decrement value
           if(c < min || c == 255)c = max;          		// If less than min, rollover to max
        }
        ByteToStr(c, Txt);                          		// Convert selected value to string
        Lcd_Out(2,1,Txt);                             		// Display selected value on LCD
     } 
     Delay_Ms(10);
     while(MODE == 0);                             		// Wait until MODE button is released
     return c;                                     		// return number to calling program
}


//
// This function sets the RTC with the new Date and Time values. The number to be sent to
// the RTC chip is divided by 10, shifted left 4 digits, and the remainder is added to it. Thus,
// for example if the number is decimal 25, it is converted into bit pattern "0010 0101" and
// stored at the appropriate RTC memory
//
void Set_RTC()
{
//
// Convert Date and Time into a format compatible with the RTC chip
//
    seconds = ((newseconds / 10) << 4) + (newseconds % 10);
    minutes = ((newminutes / 10) << 4) + (newminutes % 10);
    hours = ((newhour / 10) << 4) + (newhour % 10);
    month = ((newmonth / 10) << 4) + (newmonth % 10);
    day = (newyear << 6) + ((newday / 10) << 4) + (newday % 10);
     
    I2C1_Start();              				// Send START bit to RTC chip
    I2C1_Wr(0xA0);             				// Address the RTC chip
    I2C1_Wr(0x00);             				// Start from address 0
    I2C1_Wr(0x80);             				// Pause RTC counter 
    I2C1_Wr(0x00);             				// Write to hundreths memory location
    I2C1_Wr(seconds);          				// Write to seconds memory location
    I2C1_Wr(minutes);          				// Write to minutes memory location
    I2C1_Wr(hours);            				// Write to hours memory location;
    I2C1_Wr(day);              				// Write to year/day memory location
    I2C1_Wr(month);            				// Write to month memory location
    I2C1_Stop();               				// Send STOP bit
    
    I2C1_Start();              				// Send START bit to RTC chip
    I2C1_Wr(0xA0);             				// Address the RTC chip
    I2C1_Wr(0);                				// Start from address 0 (Configuration reg)
    I2C1_Wr(0);                				// Write 0 to Conf reg to start counter
    I2C1_Stop();               				// Send STOP bit
}



void main()
{
    ANSELB = 0;                              		// Configure PORTB as digital
    ANSELC = 0;                              		// Configure PORTC as digital
    ANSELD = 0;                              		// Configure PORTD as digital
    TRISD.RD0 = 1;                          		// Configure RD0 (MODE)as input
    TRISD.RD1 = 1;                           		// Configure RD1 (UP) as input
    TRISD.RD2 = 1;                           		// Configure RD2 (DOWN) as input
    
    Lcd_Init();                              		// Initialize LCD
    Lcd_Cmd(_LCD_CURSOR_OFF);                		// Disable cursor
    Lcd_Cmd(_LCD_CLEAR);                     		// Clear LCD
    
    I2C1_Init(100000);                        		// Initialize I2C module
//
// If the MODE button is pressed on startup we must get into SETUP phase
//
    if(MODE == SETUP)                       			// If SETUP mode
    {
        while(MODE == SETUP);                  			// Wait until MODE button is released
        newday = Set_Date_Time("DAY:",1,31);                    // Get current day
        newmonth = Set_Date_Time("MONTH:", 1,12);        	// Get current month
        newyear = Set_Date_Time("YEAR (201x):", 3,6);       	// Get current year (201x)
        newhour = Set_Date_Time("HOUR:", 0, 23);             	// Get current hour
        newminutes = Set_Date_Time("MINUTES:", 0, 59);  	// Get current minutes
        newseconds = Set_Date_Time("SECONDS:", 0, 59);          // Get current seconds
        //
        // We have got all the new Date and Time values. Now set the RTC with these values
        //
        Set_RTC();
    }

//
// Read the Date and Time from the RTC chip and display on the LCD in the following format:
// Row 1: dd-mm-yyyy
// Row 2: hh:mm:ss
//
    Lcd_Out(1, 1, "dd-mm-2013");
    Lcd_Out(2, 1, "hh:mm:ss");
    while(1)
    {
        Read_Date_Time();                       		// Read Date and Time from RTC chip
        Convert_Date_Time();                    		// Convert into a form to display
        Display_Date_Time();                    		// Display Date and Time on the LCD
    }
}
