/******************************************************************************
                     		Dual 7-SEGMENT DISPLAY EVENT COUNTER
                     		===================================

In this project two common cathode 7-segment LED displays are connected to PORTD of a
PIC18F45K22 microcontroller and the microcontroller is operated from an 8MHz crystal.
Digit 1 (right digit) enable pin is connected to port pin RA0 and digit 2 (left digit) enable pin
is connected to port pin RA1 of the microcontroller.

The program is an event counter. 3 push-button switches are connected to PORTB as follows:

  RB0   Event push button switch
  RB1   Stop_Count push button switch
  RB2   Clear_Count push button switch

Events are assumed to occur when RB0 goes from 1 to 0. When an event occurs, variable Cnt
is incremented by 1. The Display is refreshed every 5ms and the count is displayed. Pressing
Stop Count button disables the counting. Pressing the Clear Count button clears the count to 0.

This program uses co-operative multitasking, implemented using a switch statement. There are
4 tasks in the program:

TASK1: Refreshes the Display every 5ms
TASK2: Disables counting
TASK3: Clears counting to zero
TASK4: Increments counting when an event is detected

Author:   Dogan Ibrahim
Date:     October 2013
File:     MIKROC-RTOS1.C
*******************************************************************************/
#define DIGIT1 PORTA.RA0                    		// Display DIGIT 1 enable
#define DIGIT2 PORTA.RA1                    		// Display DIGIT 2 enable
#define EVENT PORTB.RB0                     		// Event input (push button)
#define STOP_COUNT PORTB.RB1                		// STOP_COUNT push button
#define CLEAR_COUNT PORTB.RB2               		// CLEAR_COUNT push button

unsigned char state = 1;                    		// Initial value of state variable
unsigned char Count_Flag = 1;               		// Set when counting is enables
unsigned char refresh = 0;
unsigned char Cnt = 0;                      		// Initial value of count
unsigned int Timer = 5;                     		// Refreshing time (ms)
unsigned char Sbutton = 0;

//
// Generate Timer interrupts every milliseconds
//
void interrupt()
{
  TMR0L = 225;                                		// Re-load Timer0 for 1ms interrupts
  Timer--;                                    		// Decrement variable Timer
  INTCON = 0x20;                              		// Re-enable Timer0 interrupts
}

//
// This function finds the bit pattern to be sent to the port to display a number
// on the 7-segment LED. The number is passed in the argument list of the function.
//
unsigned char Display(unsigned char no)
{
    unsigned char Pattern;
    unsigned char SEGMENT[] = {0x3F,0x06,0x5B,0x4F,0x66,0x6D,
                               0x7D,0x07,0x7F,0x6F};

    Pattern = SEGMENT[no];                             	// Pattern to return
    return (Pattern);
}

//
// This task refreshes the 7-segment LEDs every 5 milliseconds
//
void TASK1()
{
      unsigned char Msd, Lsd;
      
         if(Timer == 0)                             		// If 5ms has elapsed
         {
           Timer = 5;                               		// Re-load variable Timer
           if(refresh == 0)                         		// Time to refresh MSD digit ?
           {
              refresh = 1;
              DIGIT1 = 0;
              Msd = Cnt / 10;                                   // MSD digit
              PORTD = Display(Msd);                        	// Send to PORTD
              DIGIT2 = 1;                                       // Enable digit 2
           }
           else if(refresh == 1)                             	// Time to refresh LSD digit ?
           {
              refresh = 0;
              DIGIT2 = 0;                                       // Disable digit 2
              Lsd = Cnt % 10;                                   // LSD digit
              PORTD = Display(Lsd);                             // Send to PORTD
              DIGIT1 = 1;                                       // Enable digit 1
           }
         }
}


//
// This task stops the count
//
void TASK2()
{
      if(STOP_COUNT == 0)Count_Flag = 0;                  	// Clear count flag to stop counting
}

//
// This task clears the count
//
void TASK3()
{
      if(CLEAR_COUNT == 0)Cnt = 0;                    		// Clear the count to 0
}

//
// This task incrfements the event counter
//
void TASK4()
{
      if(EVENT == 0 && Count_Flag == 1)Sbutton = 1;         	// If event and counting enabled
      if(EVENT == 1 && Sbutton == 1 && Count_Flag == 1)     	// If event has been removed
      {
         Cnt++;                                                 // Increment count
         Sbutton = 0;
      }

}


//
// Start of MAIN Program
//
void main()
{
     ANSELA = 0;                                                 // Configure PORTA as digital
     ANSELB = 0;                                   		 // Configure PORTB as digital
     ANSELD = 0;                                                 // Configure PORTD as digital
     TRISA = 0;                                               	 // Configure PORTA as outputs
     TRISB = 7;                            			 // Configure RB0, RB1, RB2 as input
     TRISD = 0;                                               	 // Configure PORTD as outputs

     DIGIT1 = 0;                                              	 // Disable digit 1
     DIGIT2 = 0;                                              	 // Disable digit 2
//
// Set Timer0 to interrupt at every millisecond
//

     T0CON = 0xC5;                          			 // Configure T0CON register
     TMR0L = 225;                            			 // Load TMR0L register
     INTCON = 0xA0;                           			 // Enable Timer0 interrupts
//
// Start of multitasking loop
//
     while(1)
     {
       switch(state)
       {
         case 1:
                TASK1();                        		 // Do TASK1
                state = 2;                       		 // Next task is TASK2
                break;
         case 2:
                TASK2();                        		 // Do TASK2
                state = 3;                      		 // Next task is TASK3
                break;
         case 3:
                TASK3();                      		         // Do TASK3
                state = 4;                     			 // Next task is TASK4
                break;
         case 4:
                TASK4();                      			 // Do TASK4
                state = 1;                    			 // Next task is TASK1
                break;
       }
     }
}
