/******************************************************************************
         Dual 7-SEGMENT DISPLAY EVENT COUNTER WITH ROUND-ROBIN MULTITASKING
         ==================================================================

In this project two common cathode 7-segment LED displays are connected to PORTD of a
PIC18F45K22 microcontroller and the microcontroller is operated from an 8MHz crystal.
Digit 1 (right digit) enable pin is connected to port pin RA0 and digit 2 (left digit) enable pin is
connected to port pin RA1 of the microcontroller.

The program is an event counter. 3 push-button switches are connected to PORTB as follows:

  RB0   Event push button switch
  RB1   Stop Count push button switch
  RB2   Clear Count push button switch

Events are assumed to occur when RB0 goes from 1 to 0. When an event occurs the variable
Cnt is incremented by 1. The Display is refreshed every 5ms and the count is displayed.
Pressing Stop Count button disables the counting. Pressing the Clear Count button clears
the count to 0.

This program uses round-robin multitasking algorithm with the modification that the allocated
CPU time to each task can be set by the user.

There are 4 tasks in the program:

TASK0: Refreshes the Display every 5ms
TASK1: Disables counting
TASK2: Clears counting to zero
TASK3: Increments counting when an event is detected

Author:   Dogan Ibrahim
Date:     October 2013
File:     MIKROC-RTOS2.C
*******************************************************************************/
#define MaxTsk 4                                  	// Define number of tasks
#include "RTOS.C"                                 	// Include RTOS.C file

#define DIGIT1 PORTA.RA0                                // Display DIGIT 1 enable
#define DIGIT2 PORTA.RA1                                // Display DIGIT 2 enable
#define EVENT PORTB.RB0                                 // Event input (push button)
#define STOP_COUNT PORTB.RB1                            // STOP_COUNT push button
#define CLEAR_COUNT PORTB.RB2                           // CLEAR_COUNT push button

unsigned char refresh = 0;
unsigned char Count_Flag = 1;
unsigned char Cnt = 0;

//
// This function finds the bit pattern to be sent to the port to display a number
// on the 7-segment LED. The number is passed in the argument list of the function.
//
unsigned char Display(unsigned char no)
{
    unsigned char Pattern;
    unsigned char SEGMENT[] = {0x3F,0x06,0x5B,0x4F,0x66,0x6D,
                               0x7D,0x07,0x7F,0x6F};

    Pattern = SEGMENT[no];                                            // Pattern to return
    return (Pattern);
}

//
// This task refreshes the 7-segment LEDs every 5 milliseconds
//
void TASK0()
{
      unsigned char Msd, Lsd;

      InitTask(0,1);                            		// Allocate max 1ms to TASK0
      while(1)
      {
        if(TimerA <= 0)                           		// Time to refresh ?
        { 
          TimerA = 4;                            		// Reload TimerA with 4ms
          if(refresh == 0)                                      // Time to refresh MSD digit ?
          {
            refresh = 1;
            DIGIT1 = 0;
            Msd = Cnt / 10;                                      // MSD digit
            PORTD = Display(Msd);                                // Send to PORTD
            DIGIT2 = 1;                                          // Enable digit 2
          }
          else if(refresh == 1)                                  // Time to refresh LSD digit ?
          {
            refresh = 0;
            DIGIT2 = 0;                                          // Disable digit 2
            Lsd = Cnt % 10;                                      // LSD digit
            PORTD = Display(Lsd);                                // Send to PORTD
            DIGIT1 = 1;                                          // Enable digit 1
          }
        }
        SwapTask;                                                // Return to scheduler
      }
}


//
// This task stops the count
//
void TASK1()
{                                                       	// Allocate maximun 1ms to TASK1
      InitTask(1,1);
      while(1)
      {
         while(STOP_COUNT == 1);                  		// Wait until button is pressed
         Count_Flag = 0;                          		// Clear count flag to stop counting
         SwapTask;                                		// Return to scheduler
      }
}

//
// This task clears the count
//
void TASK2()
{
      InitTask(2,1);                         			//Allocate maximum 1ms to TASK2
      while(1)
      {
         while(CLEAR_COUNT == 1);           			// Wait until button is pressed
         Cnt = 0;                           			// Clear Cnt
         SwapTask;                          			// Return to scheduler
      }
}

//
// This task increments the event counter
//
void TASK3()
{
      InitTask(3,1);                          			// Allocate maximum 1ms to TASK3
      while(1)
      {
         while(EVENT == 1);                   			// Wait until button press-release
         while(EVENT == 0);
         if(Count_Flag == 1)Cnt++;              		// Increment Cnt
         SwapTask;                               		// Return to scheduler
      }

}


//
// Start of MAIN Program
//
void main()
{
     ANSELA = 0;                                                 // Configure PORTA as digital
     ANSELB = 0;                                                 // Configure PORTB as digital
     ANSELD = 0;                                                 // Configure PORTD as digital
     TRISA = 0;                                                  // Configure PORTA as outputs
     TRISB = 7;                                                  // Configure RB0, RB1, RB2 as input
     TRISD = 0;                                                  // Configure PORTD as outputs

     DIGIT1 = 0;                                                 // Disable digit 1
     DIGIT2 = 0;                                                 // Disable digit 2
     TimerA = 4;                                                 // Set TimerA variable to 4ms
     
     TASK0();                                               	 // Initialize TASK0
     TASK1();                                               	 // Initialize TASK1
     TASK2();                                               	 // Initialize TASK2
     TASK3();                                               	 // Initialize TASK3
     StartTasks();                                          	 // Start Tasks
}
