/*******************************************************************************
                  		TEMPERATURE DATA LOGGER PROJECT
                  		===============================

In this project an SD card is connected to PORTC as follows:

   CS	RC2
   CLK	RC3
   DO	RC4
   DI	RC5

In addition, a MAX232 type RS232 voltage level converter chip is connected to serial ports RC6
and RC7. Also, a LM35DZ type analog temperature sensor is connected to analog input AN0 of the microcontroller.

The program is menu based. The user is given options of either to send the saved temperature
data to a PC, or to read and save new data on the SD card, or to read temperature data and append to the existing file. Temperature is read at every 10 seconds.

The temperature is stored in a file called "TEMPERTR.TXT"

Author:	 Dogan Ibrahim
Date: 	 September 2013
File: 	 MIKROC-SD2.C
*******************************************************************************/
// MMC module connections
sbit Mmc_Chip_Select           at LATC2_bit;
sbit Mmc_Chip_Select_Direction at TRISC2_bit;
// End of MMC module connections

char filename[] = "TEMPERTR.TXT";
unsigned short character;
unsigned long file_size,i,rec_size;
unsigned char ch1,ch2,flag,ret_status,choice;
unsigned char temperature[10],txt[12];

//
// This function sends carriage-return and line-feed to USART
//
void Newline()
{
    Uart1_Write(0x0D);                        			// Send carriage-return
    Uart1_Write(0x0A);                        			// Send line-feed
}


//
// This function sends a space character to USART
//
void Space()
{
    Uart1_Write(0x20);
}



//
// This function reads the temperature from analog input AN0
//
void Get_Temperature()
{
     unsigned long Vin, Vdec,Vfrac;
     unsigned char op[12];
     unsigned char i,j;

     Vin = Adc_Read(0);            				// Read from channel 0 (AN0)
     Vin = 488*Vin;                				// Scale up the result
     Vin = Vin /10;                     				// Convert to temperature in C
     Vdec = Vin / 100;             				// Decimal part
     Vfrac = Vin % 100;            				// Fractional part
     LongToStr(Vdec,op);           				// Convert Vdec to string in "op"
//
// Remove leading blanks
//
        j=0;
        for(i=0;i<=11;i++)
        {
          if(op[i] != ' ')            				// If a blank
          {
             temperature[j]=op[i];
             j++;
          }
        }

        temperature[j] = '.';					// Add "."
        ch1 = Vfrac / 10;                   			// fractional part
        ch2 = Vfrac % 10;
        j++;
        temperature[j] = 48+ch1;				// Add fractional part
        j++;
        temperature[j] = 48+ch2;
        j++;
        temperature[j] = 0x0D;					// Add carriage-return
        j++;
        temperature[j] = 0x0A;					// Add line-feed
        j++;
        temperature[j]='\0';
}



//
// Start of MAIN program
//
void main()
{
   unsigned char i;
   
   ANSELC = 0;                         				// Configure PORTC as digital
   ANSELA = 1;                         				// Configure RA0 as analog
   TRISA = 0x1;                        				// RA0 (AN0) is input
//
// Configure the serial port
//
   Uart1_Init(2400);
//
// Initialise the SPI bus
//
   SPI1_Init_Advanced(_SPI_MASTER_OSC_DIV64, _SPI_DATA_SAMPLE_MIDDLE,
                      _SPI_CLK_IDLE_LOW, _SPI_LOW_2_HIGH);
//
// Initialise the SD card FAT file system
//
   while(Mmc_Fat_Init());
//
// Display the MENU and get user choice
//
 while(1)
 {
   Newline();
   Newline();
   Uart1_Write_Text("TEMPERATURE DATA LOGGER");             		// Display heading on the PC
   Newline();
   Newline();
   Uart1_Write_Text("1. Send temperature data to the PC");              // Display opt 1 on the PC
   Newline();
   Uart1_Write_Text("2. Save temperature data in a new file");          // Display opt 2 on the PC
   Newline();
   Uart1_Write_Text("3. Append temperature data to an existing file");  // Display opt 3 on the PC
   Newline();
   Newline();
   Uart1_Write_Text("Choice ? ");           				// Get choice

//
// Read a character from the PC keyboard
//
   flag = 0;
   do {
    if (Uart1_Data_Ready() == 1)          				// If data received
    {
      choice = Uart1_Read();         					// Read the received data
      Uart1_Write(choice);           					// Echo received data
      flag = 1;
    }
  } while (!flag);
  Newline();
  Newline();
   rec_size = 0;

//
// Now process user choice
//
   switch(choice)
   {
        case '1':
                 ret_status = Mmc_Fat_Assign(&filename,1);
                 if(!ret_status)
                 {
                     Uart1_Write_Text("File does not exist...");
                     Newline();
                     Uart1_Write_Text("Try again...");
                 }
                 else
                 {
                   //
                   // Read the data and send to UART
                   //
                   Uart1_Write_Text("Sending saved data to the PC...");
                   Newline();
                   Mmc_Fat_Reset(&file_size);
                   for(i=0; i<file_size; i++)
                   {
                         Mmc_Fat_Read(&character);
                         Uart1_Write(character);
                   }
                   Newline();
                   Uart1_Write_Text("End of data...");
                  }
                  break;
        case '2':
                 //
                 // Start the A/D converter, get temperature readings every
                 // 10 seconds, and then save in a NEW file
                 //
                 Uart1_Write_Text("Saving data in a NEW file...");
                 Newline();
                 Mmc_Fat_Assign(&filename,0x80);       				// Assign the file
                 Mmc_Fat_Rewrite();                    				
                 Mmc_Fat_Write("TEMPERATURE DATA - SAVED EVERY 10 SECONDS\r\n",43);
                 //
                 // Read the temperature from A/D converter, format and save
                 //
                 for(i = 0; i < 100; i++)
                 {
                    Mmc_Fat_Append();
                    Get_Temperature();
                    Mmc_Fat_Write(temperature,9);
                    rec_size++;
                    LongToStr(rec_size,txt);
                    Newline();
                    Uart1_Write_Text("Saving record:");
                    Uart1_Write_Text(txt);
                    Delay_ms(10000);
                 }
                 break;
        case '3':
                 //
                 // Start the A/D converter, get temperature readings every
                 // 10 seconds, and then APPEND to the existing file
                 //
                 Uart1_Write_Text("Appending data to the existing file...");
                 Newline();
                 ret_status = Mmc_Fat_Assign(&filename,1);       		// Assign the file
                 if(!ret_status)
                 {
                     Uart1_Write_Text("File does not exist - can not append...");
                     Newline();
                     Uart1_Write_Text("Try again...");
                     Newline();
                 }
                 else
                 {
                 //
                 // Read the temperature from A/D converter, format and save
                 //
                      for(i = 0; i < 100; i++)
                      {
                          Mmc_Fat_Append();
                          Get_Temperature();
                          Mmc_Fat_Write(temperature,9);
                          rec_size++;
                          LongToStr(rec_size,txt);
                          Newline();
                          Uart1_Write_Text("Appending new record:");
                          Uart1_Write_Text(txt);
                          Delay_ms(10000);
                      }
                 }
                 break;
        default:
                Uart1_Write_Text("Wrong choice.Try again...");
   }
 }
}

