/*===========================================================================
                        TEMPERATURE AND RELATIVE HUMIDITY MEASUREMENT
                        =============================================

This projects measures both the ambient temperature and the relative humidity and then
displays the readings on an LCD.

The SHT11 single chip temperature and relative humidity sensor is used in this project. The
sensor is connected as follows to a PIC18F45K22 type microcontroller, operating at 8 MHz:

  Sensor   Microcontroller Port
  DATA           RC4
  SCK            RC3

A 10K pull-up resistor is used on the DATA pin. In addition, a 100nF decoupling capacitor is
used between the VDD and the GND pins. The sensor is operated from a +5V supply.

The connections between the LCD and the microcontroller is as in the earlier LCD based projects.

Author:  Dogan Ibrahim
Date:    September 2013
File:    MIKRO-SHT11.C
============================================================================*/
// LCD module connections
sbit LCD_RS at RB4_bit;
sbit LCD_EN at RB5_bit;
sbit LCD_D4 at RB0_bit;
sbit LCD_D5 at RB1_bit;
sbit LCD_D6 at RB2_bit;
sbit LCD_D7 at RB3_bit;

sbit LCD_RS_Direction at TRISB4_bit;
sbit LCD_EN_Direction at TRISB5_bit;
sbit LCD_D4_Direction at TRISB0_bit;
sbit LCD_D5_Direction at TRISB1_bit;
sbit LCD_D6_Direction at TRISB2_bit;
sbit LCD_D7_Direction at TRISB3_bit;
// End LCD module connections

//SHT11 connections
sbit SHT11_SDA at RC4_bit;                                 		// SHT11 DATA pin
sbit SHT11_SCK at RC3_bit;                                 		// SHT11 SCK pin
sbit SHT11_SDA_Direction at TRISC4_bit;                    		// DATA pin direction
sbit SHT11_SCK_Direction at TRISC3_bit;                    		// SCK pin direction
//
// SHT11 Constants for calculating humidity (in 12 bit mode)
//
const float C1 = -2.0468;                                             	// -2.0468
const float C2 = 0.0367;                                             	// 0.0367
const float C3 = -1.5955E-6;                                   		// -1.5955* 10^-6
//
// SHT11 Constants for relative humidity temperature correction (in 12 bit mode)
//
const float t1 = 0.01;                                                  // 0.01
const float t2 = 0.00008;                                       	// 0.00008
//
// SHT11 temperature conversion coefficients (14 bit mode)
//
const float d1 = -40.1;                                            	// -40.1
const float d2 = 0.01;                                             	// 0.01

unsigned char i, mode;
unsigned int buffer;
float Res, Ttrue, RHtrue;
char T[] = "T=            ";
char H[] = "H=            ";

//
// Function to send the Transmission_Start_Sequence
//
void Transmission_Start_Sequence(void)
{
  SHT11_SDA_Direction = 1;                                             	// Set SDA as input
  SHT11_SCK = 1;                                                       	// SCK HIGH
  Delay_us(1);                                                   	// 1us delay
  SHT11_SDA_Direction = 0;                                             	// Set SDA as output
  SHT11_SDA = 0;                                                       	// SDA LOW
  Delay_us(1);                                                   	// 1us delay
  SHT11_SCK = 0;                                                        // SCK LOW
  Delay_us(1);                                                   	// 1us delay
  SHT11_SCK = 1;                                                       	// SCK HIGH
  Delay_us(1);                                                   	// 1us delay
  SHT11_SDA_Direction = 1;                                              // Set SDA as input
  Delay_us(1);                                                   	// 1us delay
  SHT11_SCK = 0;                                                       	// SCK low
}

//
// This function sends the Reset_Sequence
//
void Reset_Sequence()
{
      SHT11_SCK = 0;                                                    // SCL low
      SHT11_SDA_Direction = 1;                                          // Define SDA as input
      for (i = 1; i <= 10; i++)                                      	// Repeat 10 times
  {
      SHT11_SCK = 1;                                                	// Send clock pulses
      Delay_us(1);
      SHT11_SCK = 0;
      Delay_us(1);
  }
  Transmission_Start_Sequence();
}


//
// This function sends ACK
//
void Send_ACK()
{
  SHT11_SDA_Direction = 0;                                     		// Define SDA as output
  SHT11_SDA = 0;                                               		// SDA low
  SHT11_SCK = 1;                                               		// SCL high
  Delay_us(1);                                           		// 1us delay
  SHT11_SCK = 0;                                               		// SCL low
  Delay_us(1);                                           		// 1us delay
  SHT11_SDA_Direction = 1;                                     		// Define SDA as input
}


//
// This function returns temperature or humidity depending on the argument
//
float Measure(unsigned char command)
{
   mode = command;                                           		// Mode is 3 or 5
   Reset_Sequence();                                       		// Reset SHT11
   Transmission_Start_Sequence();                              		// Start transmission sequence 

   SHT11_SDA_Direction = 0;                                 		// Set SDA as output
   SHT11_SCK = 0;                                           		// Set SCK as LOW
//
// Send address and command to SHT11 sensor. A total of 8 bits are sent
//
   for(i = 0; i < 8; i++)                                          	// Send address and command
   {
      if (mode.F7 == 1)SHT11_SDA_Direction = 1;             		// If MSB (bit 7) is 1, Set SDA to 1
      else                                                       				// If MSB is 0
      {                                                          				// else MSB is 0
         SHT11_SDA_Direction = 0;                                      	// define SDA as output
         SHT11_SDA = 0;                                                	// Set SDA to 0
      }
      Delay_us(1);                                                     	// 1us delay
      SHT11_SCK = 1;                                                 	// SCL high
      Delay_us(1);                                                     	// 1us delay
      SHT11_SCK = 0;                                                 	// SCL low
      mode = mode << 1;                                         	// Move contents of j one place left
    }
//
// Give a SCK pulse for the ACK
//
   SHT11_SDA_Direction = 1;                                         	// Set SDA to input (to read ACK)
   SHT11_SCK = 1;                                                   	// SCL high
   Delay_us(1);                                                       	// 1us delay
   SHT11_SCK = 0;                                                   	// SCL low
   Delay_us(1);                                                       	// 1us delay
//
// Now wait until the measurement is ready (SDA goes LOW when data becomes ready)
//
   while (SHT11_SDA == 1)Delay_us(1);                          		// Wait until SDA goes LOW
//
// Now, the data is ready, read the data as 2 bytes. Read all 16 bits even though the
// upper nibble may not be relevant
//
  buffer = 0;
  for (i = 1; i <=16; i++)                                          	// DO 16 tim
  {
      buffer = buffer << 1;                                         	// Move MSB one place left
      SHT11_SCK = 1;                                                 	// SCK HIGH
      if (SHT11_SDA == 1)buffer = buffer | 0x0001;                 	// Get the bit as 1 (OR with data)
      SHT11_SCK = 0;
      if (i == 8)Send_ACK();                                      	// If counter i = 8 then send ACK
  }

//
// Now make the corrections to the measured value. If mode=3 then temperature, if on the
// other hand, mode=5 then relative humidity
//
  if(command == 0x03)                                        		// Temperature correction
     Res = d1 + d2*buffer;
  else if(command == 0x05)                                		// Relative humidity correction
  {
     Res = C1 + C2*buffer + C3*buffer*buffer;
     Res = (Ttrue - 25)*(t1 + t2*buffer) + Res;
  }
  return Res;                                                		// Return temperature or humidity
}


//
// This is the SHT11 startup delay (20ms)
//
void SHT11_Startup_Delay()
{
 Delay_ms(20);
}



//
// Start of MAIN program
//
void main()
{
  ANSELB = 0;                                                		// Configure PORT B as digital
  ANSELC = 0;                                                		// Configure PORT C as digital
  TRISB = 0;
  TRISC = 0;
  SHT11_SCK_Direction = 0;                                 		// SCL is output

  LCD_Init();                                                		// Initialise LCD
  Lcd_Cmd(_LCD_CURSOR_OFF);                                		// Disable cursor
  SHT11_Startup_Delay();                                		// SHT11 startup delay

  for(;;)                                                          				// DO FOREVER
  {
    SHT11_SCK_Direction = 0;                                       	// Define SCL1 as output
    Ttrue = Measure(0x03);                                		// Measure Temperature
    RHtrue = Measure(0x05);                                    		// Measure Relative humidity
    SHT11_SCK_Direction = 1;                                       	// Define SCK as input
    FloatToStr(Ttrue, T+3);                                             // Convert temperature to string
    FloatToStr(RHtrue, H+3);                                        	// Convert rel humidity to string

    Lcd_Cmd(_LCD_CLEAR);                                                // Clear display
    T[8] = 178;                                                      	// Insert Degree sign
    T[9] = 'C';                                                              			// Insert C
    T[10] = 0x0;                                                       	// Terminate with NULL
    H[8] = '%';                                                       	// Insert %
    H[9]=0x0;                                                          	// Terminate with NULL
    Lcd_Out(1,1,T);							// Display temperature
    Lcd_Out(2,1,H);							// Display humidity
    Delay_ms(1000);                                                  	// Delay 1 second
  }
}
