/******************************************************************************
                        	COMPLEX UNIPOLAR STEPPER MOTOR ROTATION
                        	========================================

In this project an UAG2 model unipolar stepper motor is connected to PORTB of a PIC18F45K22
microcontroller through a UCN5804B type motor controller chip

The program rotates the motor clockwise, or anticlockwise as requested with the required
amount of Delay between each rotation.

In this project the motor is rotated as follows:

Turn 200 revolutions clockwise
Wait 5 seconds
Turn 50 revolutions anticlockwise
Wait 3 seconds
Turn 100 revolutions clockwise
Wait 1 second
Stop

Author:   Dogan Ibrahim
Date:     October 2013
File:     MIKROC-USTP2.C
*******************************************************************************/
#define STEP PORTB.RB0                               	// UCN5804B Step input
#define DIRECTION PORTB.RB7                          	// UCN5804B Direction input

const unsigned char Step_Size = 18;                   	// Motor Step Size (degrees)
const unsigned int RPM = 500;                         	// Required RPM

unsigned int Step_Count, Delay;

//
// This function operates the stepper motor as requested. "revcnt" is the rev count, "revdir"
// is the rev direction, and "gapdly" is the intergap Delay
// If the rev count is negative the motor rotates in the required direction continuously
// if rev is equal to 0 then the motor stops
//
void Stepper_Motor(int revcnt, unsigned char revdir, unsigned int gapdly)
{
   unsigned int k,p;
   
   if(revcnt < 0)
   {
      DIRECTION = revdir;
      for(;;)
      {
         STEP = 1;					// Send pulse to the motor
         STEP = 0;
         VDelay_Ms(Delay);
      }
   }
   else
   {
      p = revcnt*Step_Count;
      DIRECTION = revdir;
      for(k = 0; k < p; k++)
      {
         STEP = 1;					// Send pulse to the motor
         STEP = 0;
         VDelay_Ms(Delay);
      }
      for(k = 0; k < gapdly; k++)Delay_Ms(1);		// Inter command delay
   }
}

//
// This is the main program. The required Rev_Count, Rev_Direction, and Inter_Delay
// must be specified here. In this example, the motor rotates 200 revs clockwise, then
// waits 5s, rotates 50 revs anticlockwise, waits 3s, rotates 100revs clockwise, waits
// 1s and then stops
//
void main()
{
     int Rev_Count[] = {200, 50, 100, 0};                    	// 0 is the terminator
     unsigned char Rev_Direction[] = {0, 1, 0};              	// 0=clockwise, 1=anticlockwise
     unsigned char Inter_Delay[] = {5000, 3000, 1000};       	// Delay in ms
     
     unsigned char j;
     
     ANSELB = 0;                                      		// Configure PORTB as digital
     TRISB = 0;                                       		// Configure PORTB as digital
     STEP = 0;                                         		// Step = 0 to start with
     
     j = 0;
     Delay = 3000/RPM;                           		// Delay after each command
     Step_Count = 360/Step_Size;
     
     while(Rev_Count[j] != 0)
     {
          Stepper_Motor(Rev_Count[j], Rev_Direction[j], Inter_Delay[j]);
          j++;
     }
     
     while(1);                                        		// End. Wait here forever
}
