/******************************************************************************
                        	Sawtooth Waveform Generation
                        	===========================

This project shows how a sawtooth waveform with specified frequency can be generated using a microcontroller. The PIC18F45K22 microcontroller is used with an 8MHz crystal.

The generatd sawtooth waveform consists of 11 steps from 0V to +5V and has a frequency of
100Hz (period of 10ms).

The MCP4921 DAC chip is used to convert the generated signal into analog. This is a 12-bit
converter controlled with the SPI bus signals.

This version of the program uses Timer0 interrupts to generate the waveform with the specified frequency.

Programmer:     Dogan Ibrahim
Date:           September 2013
File:           MIKROC-WAVE2.C

*******************************************************************************/
// DAC module connections
sbit Chip_Select at RC0_bit;
sbit Chip_Select_Direction at TRISC0_bit;
// End DAC module connections

float Sample = 0.0;

//
// This function sends 12 bits digital data to the DAC. The data is passed
// through the function argument called "value"
//
void DAC(unsigned int value) 
{
  char temp;

  Chip_Select = 0;                       		// Enable DAC chip

  // Send High Byte
  temp = (value >> 8) & 0x0F;            		// Store bits 8:11 to temp
  temp |= 0x30;                          		// Define DAC setting (choose 1x gain)
  SPI1_Write(temp);                      		// Send high byte via SPI

  // Send Low Byte
  temp = value;                       			// Store bits 0:7 to temp
  SPI1_Write(temp);                      		// Send low byte via SPI

  Chip_Select = 1;                       		// Disable DAC chip
}

//
// Timr interrupt service routine. Program jumps here at every 10ms
//
void interrupt (void)
{     
       unsigned int DAC_Value;
       
       TMR0L = 28;                                      // Re-load timer register
       INTCON.TMR0IF = 0;                                // Clear timer interrupt flag
//
// Generate the Sawtooth waveform
//
        DAC_Value = Sample * 4095;
        DAC(DAC_Value);                   		// Send to DAC converter
        Sample = Sample + 0.1;           		// Next sample
        if(Sample > 1.0)Sample = 0.0;
}


void main()
{
     ANSELC = 0;                         		// Configure PORTC as digital
     Chip_select = 1;                    		// Disable DAC
     Chip_Select_Direction = 0;         		// Set CS as output
     SPI1_Init();                        		// Initialize SPI module
//
 // Configure TIMER0 interrupts
 //
     T0CON = 0xC2;                                      // TIMER0 in 8-bit mode
     TMR0L = 28;                                       	// Load Timer register
     INTCON = 0xA0;                                     // Enable global and Timer0 interrupts
     for(;;)                                            // Wait for interrupts
     {
     }
}
