/******************************************************************************
	                    	Arbitrary Waveform Generation
                    	    	=============================

This project shows how an arbitrary waveform with specified frequency can be generated using
a microcontroller. The PIC18F45K22 microcontroller is used with an 8MHz crystal.

The generated waveform is first drawn, the waveform points extracted and stored in a floating
point array. In this example the period of the Waveform is 20ms, defined using 20 Waveform
points.

The MCP4921 DAC chip is used to convert the generated signal into analog.  This is a 12-bit
converter controlled with the SPI bus signals.

This version of the program uses Timer0 interrupts to generate the waveform with the specified frequency.

Programmer:     Dogan Ibrahim
Date:           September 2013
File:           MIKROC-WAVE4.C

*******************************************************************************/
// DAC module connections
sbit Chip_Select at RC0_bit;
sbit Chip_Select_Direction at TRISC0_bit;
// End DAC module connections

unsigned char Sample = 0;
//
// Store the waveform points in an array
//
float Waveform[] = {0, 0.625, 1.250, 1.875, 2.5, 3.125, 3.750, 4.375, 5, 5,
                    5, 5, 5, 4.375, 3.750, 3.125, 2.5, 1.875, 1.250, 0.625};

//
// This function sends 12 bits digital data to the DAC. The data is passed
// through the function argument called "value"
//
void DAC(unsigned int value)
{
  char temp;

  Chip_Select = 0;                       			// Enable DAC chip

  // Send High Byte
  temp = (value >> 8) & 0x0F;            			// Store bits 8:11 to temp
  temp |= 0x30;                          			// Define DAC setting (choose 1x gain)
  SPI1_Write(temp);                      			// Send high byte via SPI

  // Send Low Byte
  temp = value;                       				// Store bits 0:7 to temp
  SPI1_Write(temp);                      			// Send low byte via SPI

  Chip_Select = 1;                       			// Disable DAC chip
}

//
// Timr interrupt service routine. Program jumps here at every 10ms
//
void interrupt (void)
{
       unsigned int DAC_Value;

       TMR0L = 6;                                      		// Re-load timer register
       INTCON.TMR0IF = 0;                                	// Clear timer interrupt flag
//
// Generate the arbitrary waveform
//
        DAC_Value = Waveform[Sample] * 4095 / 5;
        DAC(DAC_Value);                   			// Send to DAC converter
        Sample = Sample++;           				// Next Sample
        if(Sample == 20)Sample = 0;
}


void main()
{
     ANSELC = 0;                         			// Configure PORTC as digital
     Chip_select = 1;                    			// Disable DAC
     Chip_Select_Direction = 0;         			// Set CS as output
     SPI1_Init();                        			// Initialize SPI module
//
 // Configure TIMER0 interrupts
 //
     T0CON = 0xC2;                                      	// TIMER0 in 8-bit mode
     TMR0L = 6;                                       		// Load Timer register
     INTCON = 0xA0;                                     	// Enable global and Timer0 interrupts
     for(;;)                                            	// Wait for interrupts
     {
     }
}
