/******************************************************************************
                             	GENERATE SINE WAVE
                            	==================

In this project a DAC is connected to the microcontroller output and a 50Hz sine wave with an amplitude of 2V and an offset of 2.5V is generated in real-time at the output of the DAC.

The MCP4921 12-bit DACis used in the project. This converter can operate from +3.3V to +5V,
with a typical conversion time of 4.5 microseconds. Operation of the DAC is based on the
standard SPI interface.

mikroC PRO for PIC trigonometric "sin" function is used to calculate the sine points.
50Hz waveform has the period T = 20ms, or, T = 20,000us. If we take 100 points to sample
the sine wave, then each Sample occurs at 200us. Therefore, we need a timer interrupt service routine that will generate interrupts at every 200us, and inside this routine we will calculate a
new sine point and send it to the DAC. The result is that we will get a 50Hz sine wave. Because
the DAC is unipolar, we have to shift the output waveform to a level greater than its maximum negative value so that the waveform is always positive and can be output by the DAC.


Author:      Dogan Ibrahim
Date:        September 2013
File:        MIKROC-WAVE5.C
********************************************************************************/
// DAC module connections
sbit Chip_Select at RC0_bit;
sbit Chip_Select_Direction at TRISC0_bit;
// End DAC module connections

#define T 100                                            	// 100 samples
#define R 0.0628                                         	// 2*PI/T
#define Amplitude 1638                                   	// 2V*4096/5V
#define offset 2048                                      	//2.5*4096/5V

unsigned char temp, Count = 0;
float Sample;
unsigned int Value;
float sins[100];

//
// This function sends 12 bits digital data to the DAC. The data is passed
// through the function argument called "value"
//
void DAC(unsigned int value)
{
  char temp;

  Chip_Select = 0;                                       	// Enable DAC chip

  // Send High Byte
  temp = (value >> 8) & 0x0F;                           	// Store bits 8:11 to temp
  temp |= 0x30;                                          	// Define DAC setting (choose 1x gain)
  SPI1_Write(temp);                                      	// Send high byte via SPI

  // Send Low Byte
  temp = value;                                              	// Store bits 0:7 to temp
  SPI1_Write(temp);                                      	// Send low byte via SPI

  Chip_Select = 1;                                       	// Disable DAC chip
}

//
// Timer ISR. Program jumps here at every 200 microseconds
//
void interrupt (void)
{
       TMR0L = 206;                                      	// Re-load timer register
//
// Get sine wave samples and send to DAC
//
       Value = sins[Count];
       DAC(Value);                                   		// Send to DAC converter
       Count++;
       if(Count == 100)Count = 0;
       INTCON.TMR0IF = 0;                                	// Clear timer interrupt flag
}


void main()
{
     unsigned char i;
     ANSELC = 0;                                         	// Configure PORTC as digital
     Chip_select = 1;                                    	// Disable DAC
     Chip_Select_Direction = 0;                         	// Set CS as output
     SPI1_Init();                                        	// Initialize SPI module
//
// Generate the sine wave samples offline and load into an array called sins
//
     for(i = 0; i < 100; i++)sins[i] = offset+Amplitude*sin(R*i);
//
 // Configure TIMER0 interrupts
 //
     T0CON = 0xC2;                                      	// TIMER0 in 8-bit mode
     TMR0L = 206;                                               // Load Timer register
     INTCON = 0xA0;                                     	// Enable interrupts
     for(;;)                                            	// Wait for interrupts
     {
     }
}
