/*******************************************************************************
                                BLUETOOTH COMMUNICATION
                                ==========================

This project is about using Bluetooth communication in a project. In this project a Bluetooth
module is used in slave mode. A PC is used in master mode. Messages sent to the slave module
are displayed on an LCD.

The Easy Bluetooth board (www.mikroe.com) is used in this project, connected to PORTC of
an EasyPIC V7 development board. An LCD is connected to PORTB of the microcontroller as in
the previous MPLAB XC8 projects.

Author:   Dogan Ibrahim
Date:     September 2013
File:     XC8-BLUE1.C
*******************************************************************************/
#include <xc.h>
#include <string.h>
#include <plib/usart.h>
#include <plib/xlcd.h>
#include <stdlib.h>
#pragma config MCLRE = EXTMCLR, WDTEN = OFF, FOSC = HSHP
#define _XTAL_FREQ 8000000

#define RST PORTCbits.RC1

unsigned char CMD = 1;
unsigned char AOK = 2;
unsigned char CONN = 3;
unsigned char END = 4;

unsigned char Command_Mode, temp, Response, Data_Received;
unsigned char Buffer[6], Txt[16];
unsigned char Cnt = 0;
unsigned char i = 0;
//
// UART receive interrupt handler. In Command Mode we get the following responses:
// CMD, AOK, END, CONN. The interrupt handler returns responses in command mode
// and also stores the received data from the master
//
void interrupt isr(void)
{
  if(PIR1bits.RC1IF == 1)                 		// Is this a UART receive interrupt ?
  {
     temp = getc1USART();           			// Read the received character
     if(Command_Mode == 1 && temp != 0x0)
     {
        Buffer[Cnt] = temp;
        Cnt++;
        if(Cnt == 4)
        {
          if(Buffer[0] == 'C' && Buffer[1] == 'O' && Buffer[2] == 'N' && Buffer[3] == 'N')
          {
             Response = CONN;
             Cnt = 0;
          }
        }

        if(Cnt == 5)
        {
          Cnt = 0;
          Response = 0;
          if(Buffer[0] == 'C' && Buffer[1] == 'M' && Buffer[2] == 'D' && Buffer[3] == 0x0D &&
              Buffer[4] == 0x0A)Response = CMD;
          if(Buffer[0] == 'A' && Buffer[1] == 'O' && Buffer[2] == 'K' && Buffer[3] == 0x0D &&
              Buffer[4] == 0x0A)Response = AOK;
          if(Buffer[0] == 'E' && Buffer[1] == 'N' && Buffer[2] == 'D' && Buffer[3] == 0x0D &&
              Buffer[4] == 0x0A)Response = END;
        }
     }
     else
     {
        if(temp == 0x0D)               				// If END of data
        {
           Data_Received = 1;          				// End of data received flag
           Txt[i] = 0x0;                			// Terminate data with NULL
        }
        else
        {
          Txt[i] = temp;             				// Store received data
          i++;                       				// Increment for next character
        }
     }
     PIR1bits.RC1IF = 0;           				// Clear UART interrupt flag
  }
}


//
// This function creates seconds delay. The argument specifies the delay time in seconds
//
void Delay_Seconds(unsigned char s)
{
    unsigned char i,j;

    for(j = 0; j < s; j++)
    {
        for(i = 0; i <  100; i++)__delay_ms(10);
    }
}

//
// This function creates milliseconds delay. The argument specifies the delay time in ms
//
void Delay_Ms(unsigned int ms)
{
    unsigned int i;

    for(i = 0; i <  ms; i++)__delay_ms(1);
}


//
// This function creates 18 cycles delay for the xlcd library
//
void DelayFor18TCY( void )
{
Nop(); Nop(); Nop(); Nop();
Nop(); Nop(); Nop(); Nop();
Nop(); Nop(); Nop(); Nop();
Nop(); Nop();
return;
}


//
// This fucntion creats 15ms delay for the xlcd library
//
void DelayPORXLCD( void )
{
    __delay_ms(15);
    return;
}


//
// This function creates 5ms delay for the xlcd library
//
void DelayXLCD( void )
{
    __delay_ms(5);
    return;
}

//
// This function clears the screen
//
void LCD_Clear()
{
    while(BusyXLCD());
    WriteCmdXLCD(0x01);
}

//
// This function moves the cursor to position row,column
//
void LCD_Move(unsigned char row, unsigned char column)
{
  char ddaddr = 40*(row-1) + column;
  while( BusyXLCD() );
  SetDDRamAddr( ddaddr );
}

//
// This function sends commands to the Bluetooth module. The response ic checked
// after sending a command
//
void Send_Command(const char msg[], unsigned char Resp)
{
    unsigned char i;
   
    do
    {
       i = 0;
       do
       {
           while(Busy1USART());             			// Check if UART is busy
           putc1USART(msg[i]);          			// Send to UART
           i++;
       }while(msg[i] != 0x00);          			// Until NULL terminator detected

       while(Busy1USART());
       putc1USART(0x0D);            				// Send carriage-return at the end
       Delay_Ms(500);
   }while(Response != Resp);
}



//
// Start of MAIN program
//
void main()
{
     ANSELB = 0;                      				// Configure PORTB as digital
     ANSELC = 0;                      				// Configure PORTC as digital
     TRISCbits.RC1 = 0;                  			// Configure RC1 as an output
 //
 // Enable UART receive interrupts
 //
     PIE1bits.RC1IE = 1;                   			// Clear UART1 interrupt flag
     INTCONbits.PEIE = 1;                  			// Enable UART1 interrupts
     INTCONbits.GIE = 1;                   			// Enable global interrupts

//
// Configure the LCD to use 4-bits, in multiple display mode
//
     Delay_Seconds(1);
     OpenXLCD(FOUR_BIT & LINES_5X7);
 
     Open1USART( USART_TX_INT_OFF	&       		// Initialoze UART
                 USART_RX_INT_ON	&
                 USART_ASYNCH_MODE	&
                 USART_EIGHT_BIT 	&
                 USART_CONT_RX 		&
                 USART_BRGH_LOW,
                 12);
     while(BusyXLCD());                     			// Wait if the LCD is busy
     WriteCmdXLCD(DON);                     			// Turn Display ON
     while(BusyXLCD());                     			// Wait if the LCD is busy
     WriteCmdXLCD(0x06);                    			// Move cursor right
     LCD_Clear();                                       	// Clear LCD
     LCD_Move(1,1);
     putrsXLCD("Cmd Mode");       				// Display message

     Command_Mode = 1;              				// Getting into Command Mode
     RST = 0;                     				// Reset the Bluetooth module
     Delay_Ms(100);
     RST = 1;
     Delay_Seconds(1);               				// End of Reset
     
     do                            				// Get into command mode
     {
        putc1USART('$');     					// Get into command mode
        while(Busy1USART());
        putc1USART('$');
        while(Busy1USART());
        putc1USART('$');
        Delay_Seconds(1);
     }while(Response != CMD);
     

     Send_Command("SN,BLUETOOTH2",AOK);              		// Set device name
     Send_Command("SO,SLAVE",AOK);                      	// Set extended status
     Send_Command("SM,0",AOK);                          	// Set into slave mode
     Send_Command("SA,1",AOK);                          	// Enable authentication
     Send_Command("SP,1234",AOK);                       	// Set pass code
     Send_Command("---", END);                           	// Exit command mode
     LCD_Move(1,1);
     putrsXLCD("Connecting");                          		// Display message
     while(Response != CONN);     				// Wait until connected

     Command_Mode = 0;            				// Now we are in Data mode
     Data_Received = 0;

     LCD_Move(1,1);
     putrsXLCD("Connected ");
//
// The received message is displayed inside this loop
//
     for(;;)                                              	// Endless loop
     {
        i = 0;
        while(Data_Received == 0);          			// Wait until CR is received
        Data_Received = 0;                  			// Clear data received flag
        LCD_Clear();                				// Clear LCD
        LCD_Move(1,1);
        putrsXLCD("Received Data:");                		// Display "Received Data:" on
        LCD_Move(2,1);
        putrsXLCD(Txt);                             		// Display the received data
        for(i = 0; i < 15; i++)Txt[i] = 0;           		// Clear buffer for next time
     }
}

