/******************************************************************************
                      		ULTRASONIC HUMAN HEIGHT MEASUREMENT
                      		======================================

In this project the height of a person is found and displayed on a LCD. The project uses the
PING))) Ultrasonic distance measuring transducer pair, connected to RD0 pin of a PIC18F45K22 microcontroller, operated from an 8MHz crystal.

The LCD is connected to the microcontroller as follows:

Microcontroller      	LCD
============         	===

    RB0             	D4
    RB1              	D5
    RB2              	D6
    RB3              	D7
    RB4              	E
    RB5              	R/S
    RB6              	RW
 
The program assumes that the transducer pair is mounted on a pole 200cm above the ground
level where the person stands.

Author:      Dogan Ibrahim
Date:        September 2013
File:        XC8-HEIGHT.C
********************************************************************************/

#include <xc.h>
#include <plib/xlcd.h>
#include <stdlib.h>
#pragma config MCLRE = EXTMCLR, WDTEN = OFF, FOSC = HSHP
#define _XTAL_FREQ 8000000

// Ultrasonic module connection
#define Ultrasonic PORTDbits. RD0
#define Ultrasonic_Direction TRISDbits.TRISD0
// End of Ultrasonic module connections

#define Pole_Height 200

//
// This function creates seconds delay. The argument specifies the delay time in seconds
//
void Delay_Seconds(unsigned char s)
{
    unsigned char i,j;

    for(j = 0; j < s; j++)
    {
        for(i = 0; i <  100; i++)__delay_ms(10);
    }
}

//
// This function creates 18 cycles delay for the xlcd library
//
void DelayFor18TCY( void )
{
Nop(); Nop(); Nop(); Nop();
Nop(); Nop(); Nop(); Nop();
Nop(); Nop(); Nop(); Nop();
Nop(); Nop();
return;
}


//
// This fucntion creats 15ms delay for the xlcd library
//
void DelayPORXLCD( void )
{
    __delay_ms(15);
    return;
}


//
// This function creates 5ms delay for the xlcd library
//
void DelayXLCD( void )
{
    __delay_ms(5);
    return;
}

//
// This function clears the screen
//
void LCD_Clear()
{
    while(BusyXLCD());
    WriteCmdXLCD(0x01);
}

//
// This function moves the cursor to position row,column
//
void LCD_Move(unsigned char row, unsigned char column)
{
  char ddaddr = 40*(row-1) + column;
  while( BusyXLCD() );
  SetDDRamAddr( ddaddr );
}



void main()
{
     unsigned long Tm;
     unsigned char Tl, Th;
     unsigned int h, Person_Height;
     char Txt[10];

     ANSELB = 0;                               		// Configure PORTB as digital
     ANSELD = 0;                               		// Configure PORTD as digital

     OpenXLCD(FOUR_BIT & LINES_5X7);           		 // Initialize LCD

     while(BusyXLCD());                     		// Wait if the LCD is busy
     WriteCmdXLCD(DON);                     		// Turn Display ON
     while(BusyXLCD());                     		// Wait if the LCD is busy
     WriteCmdXLCD(0x06);                    		// Move cursor right
     putrsXLCD("HEIGHT");                		// Display heading
     Delay_Seconds(2);                      		// 2 seconds delay
     LCD_Clear();                           		// Clear display
//
// Configure Timer 0 as a counter to operate in 16-bit mode with 1 microsecond
// count time. The prescaler is set to 2. The timer is stopped at this point.
//
     T0CON = 0x00;
//
// Start of program loop
// Send a pulse, start timer, get echo, stop timer, calculate distance and display
//
     for(;;)
     {
            Ultrasonic_Direction = 0;               	// RD0 in output mode
            TMR0H = 0;                              	// Clear high byte of timer
            TMR0L = 0;                              	// Clear low byte of timer

            Ultrasonic = 0;
            __delay_us(3);
            Ultrasonic = 1;                         	// Send a PULSE to Ultrasonic module
            __delay_us(5);
            Ultrasonic = 0;
            Ultrasonic_Direction = 1;               	// RD0 in input mode
            while(Ultrasonic == 0);                 	// Wait until echo is received
            T0CONbits.TMR0ON = 1;                      	// Start Timer0
            while(Ultrasonic == 1);
            T0CONbits.TMR0ON = 0;                      	// Stop Timer0
            Tl = TMR0L;                             	// Read timer low byte
            Th = TMR0H;                             	// Read timer high byte
            Tm = Th*256 + Tl;                       	// Timer as 16 bit value
            //
            // Now find the distance to person's head
            Tm = Tm / 2;                            	// Tm is half the time
            Tm = 34 * Tm;
            Tm = Tm / 1000;                         	// Divide by 1000
            h = (unsigned int)Tm;                   	// h is the distance to person's head
            Person_Height= Pole_Height - h;         	// Person's height
            //
            // Now display the height
            //
            itoa(Txt, Person_Height, 10);           	// Convert into string to display
            LCD_Clear();                        	// Clear LCD
            LCD_Move(1,1);
            putrsXLCD("Height (cm)");             	// Display heading
            LCD_Move(2,1);
            putrsXLCD(Txt);                      	// Display the height
            Delay_Seconds(1);                         	// Wait 1 second
     }
}
