/******************************************************************************
                           	Reaction Timer
                           	==============

This project is a reaction timer. Basically an LED is connected to RC0, a push-button switch is
connected to RC7 and an LCD is connected to PORTB.

The LED turns ON after a random delay (between 1 and 10 seconds) and the user is expected to
press the button as soon as he/she sees the LED. The elapsed time between the LED turning ON
and the button being pressed is displayed in milliseconds on the LCD

The project can measure reaction times from 1ms to just over 8.3 seconds (In practice it is not
possible for any person to react in more than a few seconds). A message is sent to the LCD is
the maximum reaction time is exceeded.

Programmer:     Dogan Ibrahim
Date:           September 2013
File:           XC8-REACT.C

*******************************************************************************/

#include <xc.h>
#include <plib/xlcd.h>
#include <stdlib.h>
#pragma config MCLRE = EXTMCLR, WDTEN = OFF, FOSC = HSHP
#define _XTAL_FREQ 8000000

#define LED PORTCbits.RC0
#define SWITCH PORTCbits.RC7

//
// This function creates seconds delay. The argument specifies the delay time in seconds
//
void Delay_Seconds(unsigned char s)
{
    unsigned char i,j;

    for(j = 0; j < s; j++)
    {
        for(i = 0; i <  100; i++)__delay_ms(10);
    }
}

//
// This function creates 18 cycles delay for the xlcd library
//
void DelayFor18TCY( void )
{
Nop(); Nop(); Nop(); Nop();
Nop(); Nop(); Nop(); Nop();
Nop(); Nop(); Nop(); Nop();
Nop(); Nop();
return;
}


//
// This fucntion creats 15ms delay for the xlcd library
//
void DelayPORXLCD( void )
{
    __delay_ms(15);
    return;
}


//
// This function creates 5ms delay for the xlcd library
//
void DelayXLCD( void )
{
    __delay_ms(5);
    return;
}

//
// This function clears the screen
//
void LCD_Clear()
{
    while(BusyXLCD());
    WriteCmdXLCD(0x01);
}

//
// This function moves the cursor to position row,column
//
void LCD_Move(unsigned char row, unsigned char column)
{
    char ddaddr = 40*(row-1) + column;
    while( BusyXLCD() );
    SetDDRamAddr( ddaddr );
}



void main()
{
     unsigned char T_Low, T_High, Txt[11];
     unsigned int No;
     unsigned long Cnt;

     ANSELB = 0;                                        // Configure PORTB as digital
     ANSELC = 0;                                        // Configure PORTC as digital
     TRISCbits.TRISC0 = 0;                              // Configure RC0 as output
     TRISCbits.TRISC7 = 1;                              // Configure RC7 as input

     OpenXLCD(FOUR_BIT & LINES_5X7);            	// Initialize LCD

     while(BusyXLCD());                     		// Wait if the LCD is busy
     WriteCmdXLCD(DON);                     		// Turn Display ON
     while(BusyXLCD());                     		// Wait if the LCD is busy
     WriteCmdXLCD(0x06);                    		// Move cursor right
     LCD_Clear();                           		// Clear display

     T0CON = 0x07;                              	// Timer0 for 16-bit, prescaler=256
     LED = 0;                                   	// Turn OFF LED to start with
     srand(10);                                		// Initialize random number seed
     INTCONbits.TMR0IF = 0;                     	// Clear timer overflow flag

     for(;;)
     {
//
// Generate a random number between 1 and 32767 and chenge it to be between 1 and 10
//
       No =rand();                               	// Random number between 1 and 32767
       No = No % 10 + 1;                         	// Make the number between 1 and 10
       Delay_Seconds(No);                       	// Delay No seconds
       TMR0H = 0;                                	// Clear Timer0 H register
       TMR0L = 0;                                	// Clear Timer0 L register
       LED = 1;                                  	// Turn ON LED
//
// Turn ON Timer0 (counts in multiples of 128 microseconds
//
       T0CONbits.TMR0ON = 1;                            // Turn ON Timer0
       while(SWITCH == 1);                              // Wait until the switch is pressed
//
// Switch is pressed. Stop timer and get the count
//
       T0CONbits.TMR0ON = 0;                            // Stop Timer0
       T_Low = TMR0L;                                   // Read timer low byte
       T_High = TMR0H;                                  // read timer high byte
       Cnt = (unsigned long)256*T_High + T_Low;         // Get timer count
       Cnt = 128*Cnt / 1000;                            // Convert to milliseconds

       if(INTCONbits.TMR0IF == 1)                       // If timer overflow detected
       {
           LCD_Move(1,1);
           putrsXLCD("Too long...");			// Display message
           LCD_Move(2,1);
           putrsXLCD("Try again...");
           INTCONbits.TMR0IF = 0;                       // Clear timer overflow flag
       }
       else
       {
//
// Convert count to string and display on LCD
//
          utoa(Txt, Cnt, 10);                            // Convert to string
          LCD_Move(1,1);
          putrsXLCD("Reaction (ms)");                    // Display heading
          LCD_Move(2,1);
          putrsXLCD(Txt);                                // Display reaction time in ms
        }
        LED = 0;                                         // Turn OFF LED
        Delay_Seconds(2);                                // Wait 2 seconds and repeat
        LCD_Clear();                                     // Clear LCD
     }
}
