/*****************************************************************
                  		SPI BUS BASED DIGITAL THERMOMETER
                  		=================================

In this project a TC72 type SPI bus based temperature sensor IC is used. The IC is
connected to the SPI bus pins of a PIC18F45K22 type microcontroller (i.e. to pins
RC3=SCK, RC4=SDI, and RC5=SDO) and the microcontroller is operated from an
8MHz crystal.

In addition PORT B pins of the microcontroller are connected to a standard LCD.

The microcontroller reads the temperature every second and displays on the LCD

This version of the program displays the sign as well as the fractional part of the
temperature.  An example display is:

	-23.75

Author:   Dogan Ibrahim
Date:     October 2013
File:     XC8-SPI2.C
*******************************************************************/
#include <xc.h>
#include <string.h>
#include <plib/usart.h>
#include <plib/xlcd.h>
#include <plib/spi.h>
#include <stdlib.h>

#pragma config MCLRE = EXTMCLR, WDTEN = OFF, FOSC = HSHP
#define _XTAL_FREQ 8000000

#define CE PORTCbits.RC0
#define Ready SSPSTATbits.BF

unsigned char LSB, MSB;
int result,int_part,fract_part;

//
// This function creates seconds delay. The argument specifies the delay time in seconds
//
void Delay_Seconds(unsigned char s)
{
    unsigned char i,j;

    for(j = 0; j < s; j++)
    {
        for(i = 0; i <  100; i++)__delay_ms(10);
    }
}

//
// This function creates milliseconds delay. The argument specifies the delay time in ms
//
void Delay_Ms(unsigned int ms)
{
    unsigned int i;

    for(i = 0; i <  ms; i++)__delay_ms(1);
}


//
// This function creates 18 cycles delay for the xlcd library
//
void DelayFor18TCY( void )
{
Nop(); Nop(); Nop(); Nop();
Nop(); Nop(); Nop(); Nop();
Nop(); Nop(); Nop(); Nop();
Nop(); Nop();
return;
}


//
// This fucntion creats 15ms delay for the xlcd library
//
void DelayPORXLCD( void )
{
    __delay_ms(15);
    return;
}


//
// This function creates 5ms delay for the xlcd library
//
void DelayXLCD( void )
{
    __delay_ms(5);
    return;
}

//
// This function clears the screen
//
void LCD_Clear()
{
    while(BusyXLCD());
    WriteCmdXLCD(0x01);
}


//
// Initialize the LCD, clear and home the cursor
//
void Init_LCD(void)
{
	OpenXLCD(FOUR_BIT & LINE_5X7);			// 8 bit, 5x7 character
	LCD_Clear();					// Clear LCD
}


//
// Initialize the SPI bus
//
void Init_SPI(void)
{
	OpenSPI(SPI_FOSC_4, MODE_01, SMPEND);		// SPI clk = 2MHz
}


//
// This function sends a control byte to the TC72 and waits until the
// transfer is complete
//
void Send_To_TC72(unsigned char cmd)
{
	SSPBUF = cmd;					// Send control to TC72
	while(!Ready);					// Wait until data is shifted out
}



//
// This function reads the temperature from the TC72 sensor
//
// Temperature data is read as follows:
//
// 1.  Enable TC72 (CE=1, for single byte write)
// 2.  Send Address 0x80 (A7=1)
// 3.  Clear BF flag
// 4.  Send One-Shot command (Control = 0001 0001)
// 5.  Disable TC72 (CE=0, end of single byte write)
// 6.  Clear BF flag
// 7.  Wait at leat 150ms for temperature to be available
// 8.  Enable TC72 (CE=1, for multiple data transfer)
// 9.  Send Read MSB command (Read address=0x02)
// 10.  Clear BF flag
// 11.  Send dummy output to start cloak and read data (Send 0x00)
// 12. Read high temperature into variable MSB
// 13. Send dummy output to start clock and read data (Send 0x00)
// 14. Read low temperature into variable LSB
// 15. Disable TC72 data transfer (CE=0)
// 16. Copy high result into variable "result"
void Read_Temperature(void)
{
	char dummy;

	CE = 1;						// Enable TC72
	Send_To_TC72(0x80);				// Send control write with A7=1
	dummy = SSPBUF;					// Clear BF flag
	Send_To_TC72(0x11);				// Set for one-shot operation
	CE = 0;						// Disable TC72
	dummy = SSPBUF;					// Clear BF flag
	Delay_Ms(200);					// Wait 200msfor conversion
	CE =1;						// Enable TC72
	Send_To_TC72(0x02);				// Read MSB temperature address
	dummy = SSPBUF;					// Clear BF flag
	Send_To_TC72(0x00);				// Read temperature high byte
	MSB = SSPBUF;					// save temperature and clear BF
	Send_To_TC72(0x00);				// Read temperature low byte
	LSB = SSPBUF;					// Save temperature and clear BF
	CE = 0;						// Disable TC72
	result = MSB*256+LSB;                   	// The complete temperature
}



//
// Positive and negative temperatures are displayed as well as the fractional part
//
void Format_Temperature(char *tmp)
{
        if(result & 0x8000)				// If negative
	{
		result = ~result;			// Take complement
		result++;				// Take 2's complement
		int_part = result >> 8;			// Get integer part
		*tmp++ = '-';				// Insert "-" sign
	}
	else
	{
		int_part = result >> 8;			// Get integer part
	}

	itoa(tmp,result,10);				// Convert integer to ASCII
//
// Now find the fractional part. First we must find the end of the string "tmp"
// and then append the fractional part to it
//
	while(*tmp != '\0')tmp++;			// find end of string "tmp"
//
// Now add the fractional part as ".00", ".25", ".50", or ".75"
//
	fract_part = result &0x00C0;			// fractional part
	fract_part = fract_part >> 6;			// fract is between 0-3
	switch(fract_part)
	{
		case 1:					// Fractional part = 0.25
			*tmp++ = '.';			// decimal point
			*tmp++ = '2';			// "2"
			*tmp++ = '5';			// "5"
			break;
		case 2:					// Fractional part = 0.50
			*tmp++ = '.';			// decimal point
			*tmp++ = '5';			// "5"
			*tmp++ = '0';			// "0"
                        break;
		case 3:					// Fractional part = 0.75
			*tmp++ = '.';			// decimal point
			*tmp++ = '7';			// "7"
			*tmp++ = '5';			// "5"
			break;
		case 0:					//  Fractional part = 0.00
			*tmp++ = '.';			// decimal point
			*tmp++ = '0';			// "0"
			*tmp++ = '0';			// "0"
			break;
		}
		*tmp++ = '\0';				// Null terminator
}


//
// This function clears the LCD and then displays the temperature on the LCD
//
void Display_Temperature(char *d)
{
	LCD_Clear();					// Clear LCD and home cursor
	putsXLCD(d);
}



void main(void)
{
           char msg[] = "Temperature...";
           char tmp[8];

          ANSELB = 0;                 			// Configure PORTB as digital
          ANSELC = 0;                 			// Configure PORTC as digital

          TRISCbits.RC0 = 0;				// Configure RC0 (CE) as output
         TRISB = 0;                       		// Configure PORT B as outputs

         Delay_Seconds(1);
//
// Initialize the LCD
//
         Init_LCD();
         while(BusyXLCD());                     	// Wait if the LCD is busy
         WriteCmdXLCD(DON);                     	// Turn Display ON
         while(BusyXLCD());                     	// Wait if the LCD is busy
         WriteCmdXLCD(0x06);                    	// Move cursor right
         LCD_Clear();                                   // Clear LCD
//
// Display a message on the LCD
//
	 putsXLCD(msg);
     	 Delay_Seconds(2);
	 LCD_Clear();
	 Init_SPI();        				// Initialzie the SPI bus
//
// Endless loop. Inside this loop read the TC72 temperature, display on the LCD,
// wait for 1 second and repeat the process
//
        for(;;)                          		// Endless loop
        {
	    Read_Temperature();				// Read the TC72 temperature
	    Format_Temperature(tmp);			// Format the data for display
	    Display_Temperature(tmp);			// Display the temperature
	    Delay_Seconds(1);				// Wait 1 second
	}
}
