/******************************************************************************
                        	Sawtooth Waveform Generation
                        	===========================

This project shows how a sawtooth waveform with specified frequency can be generated
using a microcontroller. The PIC18F45K22 microcontroller is used with an 8MHz crystal.

The generated sawtooth waveform consists of 11 steps from 0V to +5V and has a frequency
of 100Hz (period of 10ms).

The MCP4921 DAC chip is used to convert the generated signal into analog. This is a 12-bit
converter controlled with the SPI bus signals.

This program uses Timer0 interrupts to generate the waveform with the specified frequency.

Programmer:     Dogan Ibrahim
Date:           September 2013
File:           XC8-WAVE2.C

*******************************************************************************/

#include <xc.h>
#include <plib/spi.h>
#pragma config MCLRE = EXTMCLR, WDTEN = OFF, FOSC = HSHP
#define _XTAL_FREQ 8000000

// DAC module connections
#define Chip_Select PORTCbits.RC0
#define Chip_Select_Direction TRISCbits.TRISC0
// End DAC module connections

// SPI bus signal directions
#define SCK_Direction TRISCbits.TRISC3
#define SDO_Direction TRISCbits.TRISC5
// End of SPI bus signal directions

float Sample = 0.0;

//
// This function sends 12 bits digital data to the DAC. The data is passed through the
// function argument called "value"
//
void DAC(unsigned int value)
{
  char temp;

  Chip_Select = 0;                       			// Enable DAC chip

  // Send High Byte
  temp = (value >> 8) & 0x0F;            			// Store bits 8:11 to temp
  temp |= 0x30;                          			// Define DAC setting (choose 1x gain)
  WriteSPI1(temp);                      			// Send high byte via SPI
    
  // Send Low Byte
  temp = value;                       				// Store bits 0:7 to temp
  WriteSPI1(temp);                      			// Send low byte via SPI
  
  Chip_Select = 1;                       			// Disable DAC chip
}


//
// Timer interrupt service routine. Program jumps here at every 909 microseconds
//
void interrupt isr (void)
{
       unsigned int DAC_Value;
       
       TMR0L = 28;                                      	// Re-load timer register
       INTCONbits.TMR0IF = 0;                                	// Clear timer interrupt flag
//
// Generate the Sawtooth waveform
//
        DAC_Value = (unsigned int)(Sample * 4095);
        DAC(DAC_Value);                   			// Send to DAC converter
        Sample = Sample + 0.1;           			// Next sample
        if(Sample > 1.0)Sample = 0.0;
}


void main()
{
     ANSELC = 0;
     Chip_Select_Direction = 0;
     SCK_Direction = 0;
     SDO_Direction = 0;
     
     Chip_Select = 1;                    			// Disable DAC
     OpenSPI1(SPI_FOSC_4, MODE_00, SMPMID);               	// Initialize SPI module

//
 // Configure TIMER0 interrupts
 //
     T0CON = 0xC2;                                      	// TIMER0 in 8-bit mode
     TMR0L = 28;                                       		// Load Timer register
     INTCON = 0xA0;                                     	// Enable global and Timer0 interrupts
     for(;;)                                            	// Wait for interrupts
     {
     }
}
